@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -27,9 +27,10 @@
 #define LOGGER_FILE_SORT_MODE_TIMES 100
 #define LOGGER_FILE_SORT_MODE_NAMES_ASC 101
 #define LOGGER_FILE_SORT_MODE_NAMES_DEC 102
+#define LOGGER_FILE_SORT_MODE_NAMES_SMART 103
 
 /**
- * Returns a valid sort mode given a name: "TIMES", "NAMES_ASC", "NAMES_DEC".
+ * Returns a valid sort mode given a name: "TIMES", "NAMES_ASC", "NAMES_DEC", "NAMES_SMART".
  *  In the event of an invalid value, TIMES will be returned.
  */
 extern int loggerFileGetSortMode(const TCHAR *modeName);
@@ -47,4 +48,9 @@ extern TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode);
  */
 extern void loggerFileFreeFiles(TCHAR** files);
 
+extern TCHAR *combinePath(const TCHAR *path1, const TCHAR *path2);
+
+extern TCHAR *getRealPath(const TCHAR *path, const TCHAR *pathDesc, int errorLevel, int useQueue);
+
+TCHAR* getAbsolutePathOfFile(const TCHAR* path, const TCHAR *pathDesc, int errorLevel, int useQueue);
 #endif
