@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -14,4 +14,8 @@
 
 #include "logger_base.h"
 
+void setPrintMessageCallback(int (*callback)(const TCHAR* message));
+
+void log_printf(const TCHAR *lpszFmt, ...);
+
 #endif
