@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -54,35 +54,59 @@
 #define ENV_SOURCE_REG_ACCOUNT 16
 #endif
 
+#define PROPERTIES_DUMP_FORMAT_DEFAULT TEXT("SFNV")
+
 typedef struct EnvSrc EnvSrc;
 struct EnvSrc {
-    int     source;                 /* Source of the variable. */
-    TCHAR*  name;                   /* Name of the environment variable. */
-    EnvSrc *next;                   /* Next variable in the chain. */
+    int     source;                     /* Source of the variable. */
+    TCHAR*  name;                       /* Name of the environment variable. */
+    EnvSrc *next;                       /* Next variable in the chain. */
 };
 extern EnvSrc *baseEnvSrc;
 
 typedef struct Property Property;
 struct Property {
-    TCHAR *name;             /* The name of the property. */
-    TCHAR *value;            /* The value of the property. */
-    int finalValue;          /* TRUE if the Property can not be changed. */
-    int quotable;            /* TRUE if quotes can be optionally added around the value. */
-    int internal;            /* TRUE if the Property is internal. */
-    Property *next;          /* Pointer to the next Property in a linked list */
-    Property *previous;      /* Pointer to the next Property in a linked list */
+    TCHAR *name;                        /* The name of the property. */
+    TCHAR *value;                       /* The value of the property. */
+    TCHAR *value_inner;                 /* The value of the property (with %WRAPPER_PERCENTAGE% kept unexpanded). */
+    TCHAR *filePath;                    /* The file in which the property was declared. NULL is not defined in a conf file. */
+    int lineNumber;                     /* The line number at which the property was declared. 0 if not defined in a conf file. */
+    int definitions;                    /* Counter for the number of time the property was defined. */
+    int finalValue;                     /* TRUE if the Property can not be changed. */
+    int quotable;                       /* TRUE if quotes can be optionally added around the value. */
+    int internal;                       /* TRUE if the Property is internal. */
+    int isGenerated;                    /* TRUE if the property did not exist in the configuration and was generated using a default value. */
+    int isVariable;                     /* TRUE if the property the definition of a variable (starting with 'set.'). */
+    int lastDefinitionDepth;            /* Depth of the configuration file in which the property was previously defined, or -1 if not yet defined. */
+    Property *next;                     /* Pointer to the next Property in a linked list */
+    Property *previous;                 /* Pointer to the previous Property in a linked list */
 };
 
 typedef struct Properties Properties;
 struct Properties {
-    int debugProperties;     /* TRUE if debug information on Properties should be shown. */
-    int logWarnings;         /* Flag that controls whether or not warnings will be logged. */
-    int logWarningLogLevel;  /* Log level at which any log warnings will be logged. */
-    Property *first;         /* Pointer to the first property. */
-    Property *last;          /* Pointer to the last property.  */
-    PHashMap warnedVarMap;   /* Map of undefined environment variables for which the user was warned. */
+    int debugProperties;                /* TRUE if debug information on Properties should be shown. */
+    int exitOnOverwrite;                /* If TRUE, causes the wrapper to exit when any property is overwritten in the config files. */
+    int logLevelOnOverwrite;            /* Defines the log level of the messages reported when properties are overwritten (the value may change each time a directive is encountered as the configuration is loading). */
+    int overwrittenPropertyCausedExit;  /* Flag to keep trace whether at least one property was overridden */
+    int logWarnings;                    /* Flag that controls whether or not warnings will be logged. */
+    int logWarningLogLevel;             /* Log level at which any log warnings will be logged. */
+    int dumpLogLevel;                   /* Log level at which properties will be dumped. */
+    TCHAR* dumpFormat;                  /* Format used when dumping properties. */
+    Property *first;                    /* Pointer to the first property. */
+    Property *last;                     /* Pointer to the last property.  */
+    PHashMap warnedVarMap;              /* Map of undefined environment variables for which the user was warned. */
+    PHashMap ignoreVarMap;              /* Map of environment variables that should not be expanded. */
 };
 
+/**
+ * Get the log level of the messages reported when properties are overwritten.
+ *
+ * @param properties 
+ *
+ * @return log level 
+ */
+extern int GetLogLevelOnOverwrite(Properties *properties);
+
 /**
  * Sets an environment variable with the specified value.
  *  The function will only set the variable if its value is changed, but if
@@ -106,11 +130,18 @@ extern int setEnv(const TCHAR *name, const TCHAR *value, int source);
  *  environment variable would result in a string that is longer than bufferLength
  *  the value is truncated.
  *
+ * @param propertyValue The property value to be parsed.
+ * @param buffer output buffer where the expanded string will be copied.
+ * @param bufferLength size of the buffer.
  * @param warnUndefinedVars Log warnings about missing environment variables.
  * @param warnedUndefVarMap Map of variables which have previously been logged, may be NULL if warnUndefinedVars false.
  * @param warnLogLevel Log level at which any warnings will be logged.
+ * @param ignoreVarMap Map of environment variables that should not be expanded.
+ * @param pHasPercentage Pointer to a variable which will be set to TRUE if a %WRAPPER_PERCENTAGE% variable was found.
+ *                         - If a non-NULL pointer is passed, the variable will not be expanded and no warning will be reported.
+ *                         - If NULL is passed, the variable will be expanded to '%'.
  */
-extern void evaluateEnvironmentVariables(const TCHAR *propertyValue, TCHAR *buffer, int bufferLength, int warnUndefinedVars, PHashMap warnedUndefVarMap, int warnLogLevel);
+extern void evaluateEnvironmentVariables(const TCHAR *propertyValue, TCHAR *buffer, int bufferLength, int warnUndefinedVars, PHashMap warnedUndefVarMap, int warnLogLevel, PHashMap ignoreVarMap, int *hasPercentage);
 
 /**
  * This function returns a reference to a static buffer and is NOT thread safe.
@@ -124,6 +155,17 @@ extern TCHAR* generateTimeValue(const TCHAR* format, struct tm *timeTM);
  */
 extern TCHAR* generateRandValue(const TCHAR* format);
 
+/**
+ * Callback declaration of a function to filter lines read from the configuration file.
+ */
+typedef int (*ConfigFileReader_ReadFilterCallbackMB)(const char *bufferMB);
+
+/**
+ * Callback declaration of a function to filter lines read from the configuration file.
+ */
+typedef int (*ConfigFileReader_ReadFilterCallbackW)(const TCHAR *bufferMB);
+
+
 /**
  * Create a Properties structure loaded in from the specified file.
  *  Must call disposeProperties to free up allocated memory.
@@ -131,24 +173,29 @@ extern TCHAR* generateRandValue(const TCHAR* format);
  * @param properties Properties structure to load into.
  * @param filename File to load the properties from.
  * @param preload TRUE if this is a preload call that should have supressed error output.
- * @param argCommand Argument passed to the binary.
  * @param originalWorkingDir Working directory of the binary at the moment it was launched.
- * @param isDebugging Flag that controls whether or not debug output will be logged.
+ * @param fileRequired TRUE if the file specified by filename is required, FALSE if a missing
+ *                     file will silently fail.
+ * @param readFilterCallback Pointer to a callback funtion which will be used to filter some
+ *                           lines that should not be processed.
  *
- * @return TRUE if there were any problems, FALSE if successful.
+ * @return CONFIG_FILE_READER_SUCCESS if the file was read successfully,
+ *         CONFIG_FILE_READER_OPEN_FAIL if the file could not be found or opened.
+ *         CONFIG_FILE_READER_FAIL if there were any problems at all, or
+ *         CONFIG_FILE_READER_HARD_FAIL if the problem should cascaded all the way up.
  */
 extern int loadProperties(Properties *properties,
                           const TCHAR* filename,
                           int preload,
-                          const TCHAR *argCommand,
                           const TCHAR *originalWorkingDir,
-                          int isDebugging);
+                          int fileRequired,
+                          ConfigFileReader_ReadFilterCallbackMB readFilterCallback);
 
 /**
  * Create a Properties structure.  Must call disposeProperties to free up
  *  allocated memory.
  */
-extern Properties* createProperties();
+extern Properties* createProperties(int debug, int logLevelOnOverwrite, int exitOnOverwrite);
 
 /**
  * Free all memory allocated by a Properties structure.  The properties
@@ -165,8 +212,10 @@ extern void disposeEnvironment();
 /**
  * Remove a single Property from a Properties.  All associated memory is
  *  freed up.
+ *
+ * @return TRUE if the property was found, FALSE otherwise.
  */
-extern void removeProperty(Properties *properties, const TCHAR *propertyName);
+extern int removeProperty(Properties *properties, const TCHAR *propertyName);
 
 /**
  * Used to set a NULL terminated list of property names whose values should be
@@ -181,7 +230,7 @@ extern void setEscapedProperties(const TCHAR **propertyNames);
 
 /**
  * Returns true if the specified property matches one of the property names
- *  previosly set in a call to setEscapableProperties()
+ *  previously set in a call to setEscapedProperties()
  *
  * @param propertyName Property name to test.
  *
@@ -195,17 +244,18 @@ extern int isEscapedProperty(const TCHAR *propertyName);
  * @param properties Properties structure to add to.
  * @param filename Name of the file from which the property was loaded.  NULL, if not from a file.
  * @param lineNum Line number of the property declaration in the file.  Ignored if filename is NULL.
+ * @param depth Depth of the configuration file where the property was declared.  Ignored if filename is NULL.
  * @param propertyName Name of the new Property.
  * @param propertyValue Initial property value.
  * @param finalValue TRUE if the property should be set as static.
  * @param quotable TRUE if the property could contain quotes.
  * @param escapable TRUE if the propertyValue can be escaped if its propertyName
- *                  is in the list set with setEscapableProperties().
+ *                  is in the list set with setEscapedProperties().
  * @param internal TRUE if the property is a Wrapper internal property.
  *
  * @return The newly created Property, or NULL if there was a reported error.
  */
-extern Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum, const TCHAR *propertyName, const TCHAR *propertyValue, int finalValue, int quotable, int escapable, int internal);
+extern Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum, int depth, const TCHAR *propertyName, const TCHAR *propertyValue, int finalValue, int quotable, int escapable, int internal);
 
 /**
  * Takes a name/value pair in the form <name>=<value> and attempts to add
@@ -223,8 +273,12 @@ extern Property* addProperty(Properties *properties, const TCHAR* filename, int
  */
 extern int addPropertyPair(Properties *properties, const TCHAR* filename, int lineNum, const TCHAR *propertyNameValue, int finalValue, int quotable, int internal);
 
+extern void setInternalVarProperty(Properties *properties, const TCHAR *varName, const TCHAR *varValue, int finalValue, int ignore);
+
 extern const TCHAR* getStringProperty(Properties *properties, const TCHAR *propertyName, const TCHAR *defaultValue);
 
+extern const TCHAR* getNotEmptyStringProperty(Properties *properties, const TCHAR *propertyName, const TCHAR *defaultValue);
+
 extern const TCHAR* getFileSafeStringProperty(Properties *properties, const TCHAR *propertyName, const TCHAR *defaultValue);
 
 /**
@@ -263,10 +317,30 @@ extern int getIntProperty(Properties *properties, const TCHAR *propertyName, int
 
 extern int getBooleanProperty(Properties *properties, const TCHAR *propertyName, int defaultValue);
 
+extern int getBooleanProperties(Properties *properties, const TCHAR *propertyNameHead, const TCHAR *propertyNameTail, int all, int matchAny, TCHAR ***propertyNames, int **propertyValues, long unsigned int **propertyIndices, int defaultValue);
+
+extern void freeBooleanProperties(TCHAR **propertyNames, int *propertyValues, long unsigned int *propertyIndices);
+
+extern const TCHAR* getPropertyFilePath(Properties *properties, const TCHAR* propertyName);
+
+extern int getStatusProperty(Properties *properties, const TCHAR *propertyName, int defaultValue);
+
+extern int isGeneratedProperty(Properties *properties, const TCHAR *propertyName);
+
 extern int isQuotableProperty(Properties *properties, const TCHAR *propertyName);
 
 extern void dumpProperties(Properties *properties);
 
+/**
+ * Level at which properties will be dumped.
+ */
+extern void setPropertiesDumpLogLevel(Properties *properties, int logLevel);
+
+/**
+ * Format used when dumping properties.
+ */
+extern void setPropertiesDumpFormat(Properties *properties, const TCHAR* format);
+
 /**
  * Set to TRUE if warnings about property values should be logged.
  */
@@ -290,6 +364,4 @@ extern int propIntMax(int value1, int value2);
 /** Creates a linearized representation of all of the properties.
  *  The returned buffer must be freed by the calling code. */
 extern TCHAR *linearizeProperties(Properties *properties, TCHAR separator);
-
 #endif
-
