@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -20,11 +20,30 @@
 #ifdef WIN32
 #include <tchar.h>
 #endif
+#include "property.h"
 #include "wrapper_i18n.h"
 #include "wrapper_hashmap.h"
 
 /*#define WRAPPER_FILE_DEBUG*/
 
+/**
+ * Callback declaration which can be passed to calls to configFileReader.
+ */
+typedef int (*ConfigFileReader_Callback)(void *param, const TCHAR *fileName, int lineNumber, int depth, TCHAR *config, int exitOnOverwrite, int logLevelOnOverwrite);
+
+/* Structure used by configFileReader to read files. */
+typedef struct ConfigFileReader ConfigFileReader;
+struct ConfigFileReader {
+    ConfigFileReader_Callback callback;
+    ConfigFileReader_ReadFilterCallbackMB readFilterCallback;
+    void *callbackParam;
+    int enableIncludes;
+    int preload;
+    int debugIncludes;                  /* debugIncludes controls whether or not debug output is logged. It is set using directives in the file being read. */
+    int exitOnOverwrite;                /* If TRUE, causes the wrapper to exit when any property is overwritten in the config files. */
+    int logLevelOnOverwrite;            /* Defines the log level of the messages reported when properties are overwritten. */
+};
+
 /**
  * Tests whether a file exists.
  *
@@ -36,18 +55,16 @@ extern int wrapperFileExists(const TCHAR * filename);
 extern int wrapperGetUNCFilePath(const TCHAR *path, int advice);
 #endif
 
+extern const TCHAR* getFileName(const TCHAR* path);
+
 #ifdef WRAPPER_FILE_DEBUG
 extern void wrapperFileTests();
 #endif
 
-/**
- * Callback declaration which can be passed to calls to configFileReader.
- */
-typedef int (*ConfigFileReader_Callback)(void *param, const TCHAR *fileName, int lineNumber, TCHAR *config, int debugProperties);
-
 #define CONFIG_FILE_READER_SUCCESS   101
 #define CONFIG_FILE_READER_FAIL      102
 #define CONFIG_FILE_READER_HARD_FAIL 103
+#define CONFIG_FILE_READER_OPEN_FAIL 104
 
 /**
  * Reads configuration lines from the file `filename' and calls `callback' with the line and
@@ -58,15 +75,16 @@ typedef int (*ConfigFileReader_Callback)(void *param, const TCHAR *fileName, int
  *                     file will silently fail.
  * @param callback Pointer to a callback funtion which will be called for each line read.
  * @param callbackParam Pointer to additional user data which will be passed to the callback.
+ * @param readFilterCallback Pointer to a callback funtion which will be used to filter some
+ *                           lines that should not be processed.
  * @param enableIncludes If TRUE then includes will be supported.
  * @param preload TRUE if this is being called in the preload step meaning that all errors
  *                should be suppressed.
- * @param argCommand Argument passed to the binary.
  * @param originalWorkingDir Working directory of the binary at the moment it was launched.
  * @param warnedVarMap Map of undefined environment variables for which the user was warned.
+ * @param ignoreVarMap Map of environment variables that should not be expanded.
  * @param logWarnings Flag that controls whether or not warnings will be logged.
  * @param logWarningLogLevel Log level at which any log warnings will be logged.
- * @param isDebugging Flag that controls whether or not debug output will be logged.
  *
  * @return CONFIG_FILE_READER_SUCCESS if the file was read successfully,
  *         CONFIG_FILE_READER_FAIL if there were any problems at all, or
@@ -76,14 +94,14 @@ extern int configFileReader(const TCHAR *filename,
                             int fileRequired,
                             ConfigFileReader_Callback callback,
                             void *callbackParam,
+                            ConfigFileReader_ReadFilterCallbackMB readFilterCallback,
                             int enableIncludes,
                             int preload,
-                            const TCHAR *argCommand,
                             const TCHAR *originalWorkingDir,
                             PHashMap warnedVarMap,
+                            PHashMap ignoreVarMap,
                             int logWarnings,
-                            int logWarningLogLevel,
-                            int isDebugging);
+                            int logWarningLogLevel);
 
 #endif
 
