@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -39,6 +39,10 @@
  #define dup2 _dup2
 #endif
 #include "wrapper_i18n.h"
+#ifndef WIN32
+ #include <langinfo.h>
+#endif
+
 #include "loggerjni.h"
 #include "wrapperjni.h"
 #include "wrapperinfo.h"
@@ -53,7 +57,6 @@ int controlEventQueue[CONTROL_EVENT_QUEUE_SIZE];
 int controlEventQueueLastReadIndex = 0;
 int controlEventQueueLastWriteIndex = 0;
 
-
 /** Flag to keep track of whether StdOut has been redirected. */
 int redirectedStdOut = FALSE;
 
@@ -66,8 +69,6 @@ const char utf8ClassJavaLangString[] = {106, 97,118, 97, 47, /* java/ */
                                         83,116,114,105,110,103, 0}; /* "java/lang/String" */
 const char utf8MethodInit[] = {60,105,110,105,116, 62, 0}; /* "<init>" */
 const char utf8Sig_BrV[] = {40, 91, 66, 41, 86, 0}; /* "([B)V" */
-const char utf8Sigr_B[] = { 40, 41, 91, 66, 0}; /* "()[B") */
-const char utf8MethodGetBytes[] = {103, 101, 116, 66, 121, 116, 101, 115, 0}; /* getBytes */
 const char utf8ClassJavaLangOutOfMemoryError[] = {106, 97,118, 97, 47, /* java/ */
                                                   108, 97,110, 103, 47, /* lang/ */
                                                   79, 117, 116, 79, 102, 77, 101, 109, 111, 114, 121, 69, 114, 114, 111, 114, 0}; /* OutOfMemoryError */
@@ -99,45 +100,110 @@ char *utf8SigIStringrV;
 #endif
 
 
-
 /**
  * Cause the current thread to sleep for the specified number of milliseconds.
- *  Sleeps over one second are not allowed.
  *
  * @param ms Number of milliseconds to wait for.
+ * @param interrupt TRUE to return when nanosleep was interrupted by a signal,
+ *                  FALSE to continue sleeping the remaining time.
+ *                  This parameter is ignored on Windows or when using usleep.
  *
- * @return TRUE if the was interrupted, FALSE otherwise.  Neither is an error.
+ * @return the number of remaining ms to sleep if interrupted, -1 if there was an error, 0 otherwise.
  */
-int wrapperSleep(int ms) {
+int wrapperSleepInterrupt(int ms, int interrupt) {
+    int result;
+
 #ifdef WIN32
     Sleep(ms);
+    result = 0; /* On Windows Sleep returns void. */
 #else
     /* We want to use nanosleep if it is available, but make it possible for the
        user to build a version that uses usleep if they want.
        usleep does not behave nicely with signals thrown while sleeping.  This
        was the believed cause of a hang experienced on one Solaris system. */
-#ifdef USE_USLEEP
-    usleep(ms * 1000); /* microseconds */
-#else
+ #ifdef USE_USLEEP
+    result = wrapperUsleep(ms * 1000); /* microseconds */
+ #else
     struct timespec ts;
+    struct timespec tsRemaining;
 
     if (ms >= 1000) {
-        ts.tv_sec = (ms * 1000000) / 1000000000;
-        ts.tv_nsec = (ms * 1000000) % 1000000000; /* nanoseconds */
+        ts.tv_sec = ms / 1000;
+        ts.tv_nsec = (ms % 1000) * 1000000; /* nanoseconds */
     } else {
         ts.tv_sec = 0;
         ts.tv_nsec = ms * 1000000; /* nanoseconds */
     }
-    if (nanosleep(&ts, NULL)) {
-        if (errno == EINTR) {
-            return TRUE;
-        } else if (errno == EAGAIN) {
-            return TRUE;
+    while (((result = nanosleep(&ts, &tsRemaining)) == -1) && (errno == EINTR)) {
+        if (interrupt) {
+            return (tsRemaining.tv_sec * 1000) + (tsRemaining.tv_nsec / 1000000);
         }
+        ts.tv_sec = tsRemaining.tv_sec;
+        ts.tv_nsec = tsRemaining.tv_nsec;
     }
+ #endif
 #endif
+    return result;
+}
+
+void wrapperSleep(int ms) {
+    wrapperSleepInterrupt(ms, FALSE);
+}
+
+/**
+ * Create a jstring from a MultiBytes Char string.  The jstring must be freed up by caller.
+ *
+ * @param env The current JNIEnv.
+ * @param str The MultiBytes string to convert.
+ *
+ * @return The new jstring or NULL if there were any exceptions thrown.
+ */
+jstring JNU_NewStringFromNativeMB(JNIEnv *env, const char *str) {
+    jstring result;
+    size_t len;
+    char *strOut;
+    TCHAR* errorW;
+#ifdef WIN32
+    int encodingFrom = CP_OEMCP;
+#else
+    char* encodingFrom = nl_langinfo(CODESET);
+ #ifdef MACOSX
+    if (strlen(encodingFrom) == 0) {
+        encodingFrom = "UTF-8";
+    }
+ #endif
 #endif
-    return FALSE;
+    
+    len = strlen(str);
+    if (len > 0) {
+        if (converterMBToMB(str, encodingFrom, &strOut, __UTF8) < 0) {
+            if (strOut) {
+                /* An error message is stored in strOut (we need to convert it to wide chars to display it). */
+#ifdef WIN32
+                if (multiByteToWideChar(strOut, __UTF8, &errorW, FALSE)) {
+#else
+                if (converterMBToWide(strOut, __UTF8, &errorW, FALSE)) {
+#endif
+                    _tprintf(TEXT("WrapperJNI Warn: Unexpected conversion error: %s\n"), getLastErrorText()); fflush(NULL);
+                } else {
+                    _tprintf(TEXT("%s\n"), errorW); fflush(NULL);
+                }
+                if (errorW) {
+                    free(errorW);
+                }
+                free(strOut);
+            } else {
+                throwOutOfMemoryError(env, TEXT("JNSFNC1"));
+            }
+            return NULL;
+        }
+        result = (*env)->NewStringUTF(env, strOut);
+        free(strOut);
+    } else {
+        result = (*env)->NewStringUTF(env, str);
+    }
+    
+    return result;
 }
 
 /**
@@ -148,53 +214,51 @@ int wrapperSleep(int ms) {
  *
  * @return The new jstring or NULL if there were any exceptions thrown.
  */
-jstring JNU_NewStringNative(JNIEnv *env, const TCHAR *strW) {
+jstring JNU_NewStringFromNativeW(JNIEnv *env, const TCHAR *strW) {
     jstring result;
-
-    jclass jClassString;
-    jmethodID MID_String_init;
-    jbyteArray jBytes;
     size_t len;
     char* msgMB;
-#ifdef UNICODE
+#ifdef WIN32
     int size;
+#else
+    TCHAR* errorW;
 #endif
-#ifdef UNICODE
-    /* We need to special case empty strings as some of the functions don't work correctly for them. */
+
     len = _tcslen(strW);
     if (len > 0) {
- #ifdef WIN32
+#ifdef WIN32
         size = WideCharToMultiByte(CP_UTF8, 0, strW, -1, NULL, 0, NULL, NULL);
-        if (size == 0) {
-            /* Failed. */
+        if (size <= 0) {
             _tprintf(TEXT("WrapperJNI Warn: Failed to convert string \"%s\": %s\n"), strW, getLastErrorText()); fflush(NULL);
             return NULL;
         }
-        msgMB = malloc(sizeof(char) * (size + 1));
+        msgMB = malloc(sizeof(char) * size);
         if (!msgMB) {
             throwOutOfMemoryError(env, TEXT("JNSN1"));
             return NULL;
         }
-        WideCharToMultiByte(CP_UTF8, 0, strW, -1, msgMB, size + 1, NULL, NULL);
-        result = (*env)->NewStringUTF(env, msgMB);
-        free(msgMB);
-        return result;
- #else
-        size = wcstombs(NULL, strW, 0);
-        if (size == (size_t)-1) {
-            _tprintf(TEXT("Invalid multibyte sequence \"%s\": %s\n"), strW, getLastErrorText());
+        WideCharToMultiByte(CP_UTF8, 0, strW, -1, msgMB, size, NULL, NULL);
+#else
+        if (converterWideToMB(strW, &msgMB, MB_UTF8) < 0) {
+            if (msgMB) {
+                /* An error message is stored in msgMB (we need to convert it to wide chars to display it). */
+                if (converterMBToWide(msgMB, NULL, &errorW, FALSE)) {
+                    _tprintf(TEXT("WrapperJNI Warn: Failed to convert string \"%s\": %s\n"), strW, getLastErrorText()); fflush(NULL);
+                } else {
+                    _tprintf(TEXT("%s\n"), errorW); fflush(NULL);
+                }
+                if (errorW) {
+                    free(errorW);
+                }
+                free(msgMB);
+            } else {
+                throwOutOfMemoryError(env, TEXT("JNSN2"));
+            }
             return NULL;
         }
-
-        msgMB = malloc(sizeof(char) * (size + 1));
-        if (!msgMB) {
-            throwOutOfMemoryError(env, TEXT("JNSN2"));
-            return NULL;
-        }
-        wcstombs(msgMB, strW, size + 1);
- #endif
+#endif
     } else {
-        /* Empty string. */
+        /* We need to special case empty strings as some of the functions don't work correctly for them. */
         msgMB = malloc(sizeof(char) * 1);
         if (!msgMB) {
             throwOutOfMemoryError(env, TEXT("JNSN3"));
@@ -202,68 +266,11 @@ jstring JNU_NewStringNative(JNIEnv *env, const TCHAR *strW) {
         }
         msgMB[0] = '\0';
     }
-#else
-    msgMB = (TCHAR*)strW;
-#endif
-    result = NULL;
-    if ((*env)->EnsureLocalCapacity(env, 2) < 0) {
-        throwOutOfMemoryError(env, TEXT("JNSN4"));
-#ifdef UNICODE
-        if (msgMB) {
-            free(msgMB);
-        }
-#endif
-        return NULL; /* out of memory error */
-    }
-    len = strlen(msgMB);
-    if ((jBytes = (*env)->NewByteArray(env, (jsize)len))) {
-        (*env)->SetByteArrayRegion(env, jBytes, 0, (jsize)len, (jbyte*)msgMB);
-        if ((jClassString = (*env)->FindClass(env, utf8ClassJavaLangString))) {
-            if ((MID_String_init = (*env)->GetMethodID(env, jClassString, utf8MethodInit, utf8Sig_BrV))) {
-                result = (*env)->NewObject(env, jClassString, MID_String_init, jBytes);
-            }
-
-            (*env)->DeleteLocalRef(env, jClassString);
-        }
-
-        (*env)->DeleteLocalRef(env, jBytes);
-    }
-
-#ifdef UNICODE
-    if (msgMB) {
-        free(msgMB);
-    }
-#endif
-
+    result = (*env)->NewStringUTF(env, msgMB);
+    free(msgMB);
     return result;
 }
 
-
-#ifdef WIN32
-/* So far this function is only used by windows. if we want to use it for unix as well, first
-   provide correct wchar handling... */
-void JNU_SetByteArrayRegion(JNIEnv *env, jbyteArray* jarray, jsize start, jsize len, const TCHAR *buffer) {
-
-    char* msg;
-#if defined(UNICODE) && defined(WIN32)
-    int size;
-    size = WideCharToMultiByte(CP_OEMCP, 0, buffer, -1, NULL, 0, NULL, NULL);
-    msg = malloc(size);
-    if (!msg) {
-        throwOutOfMemoryError(env, TEXT("JSBAR1"));
-        return;
-    }
-    WideCharToMultiByte(CP_OEMCP,0, buffer,-1, msg, size, NULL, NULL);
-#else
-     msg = (TCHAR*) buffer;
-#endif
-    (*env)->SetByteArrayRegion(env, *jarray, start, len, (jbyte*) msg);
-#if defined(UNICODE) && defined(WIN32)
-    free(msg);
-#endif
-}
-#endif
-
 /**
  * Converts a jstring into a newly malloced TCHAR array.
  *
@@ -273,105 +280,70 @@ void JNU_SetByteArrayRegion(JNIEnv *env, jbyteArray* jarray, jsize start, jsize
  * @return The requested Wide String, or NULL if there was a problem.  It is
  *         the responsibility of the caller to free up the returned string.
  */
-TCHAR *JNU_GetStringNativeChars(JNIEnv *env, jstring jstr) {
-    jbyteArray jByteArrayBytes = 0;
-    jclass jClassString = NULL;
-    jmethodID jMethodIdStringGetBytes = NULL;
-#ifdef UNICODE
-    int size;
-    TCHAR* tresult;
-#endif
-    char *result = 0;
-
-    if ((*env)->EnsureLocalCapacity(env, 2) < 0) {
-        throwOutOfMemoryError(env, TEXT("GSNC1"));
-        return NULL; /* out of memory error */
-    }
-    if ((jClassString = (*env)->FindClass(env, utf8ClassJavaLangString)) != NULL) {
-        if ((jMethodIdStringGetBytes = (*env)->GetMethodID(env, jClassString, utf8MethodGetBytes, utf8Sigr_B)) != NULL) {
-            if ((jByteArrayBytes = (*env)->CallObjectMethod(env, jstr, jMethodIdStringGetBytes)) != NULL) {
-                jint len = (*env)->GetArrayLength(env, jByteArrayBytes);
-                result = (char *)malloc(sizeof(char) * (len + 1));
-                if (!result) {
-                    throwOutOfMemoryError(env, TEXT("GSNC2"));
-                } else {
-                    (*env)->GetByteArrayRegion(env, jByteArrayBytes, 0, len, (jbyte *)result);
-                    result[len] = 0; /* NULL-terminate */
-                }
-                
-                (*env)->DeleteLocalRef(env, jByteArrayBytes);
-            }
-        }
-        (*env)->DeleteLocalRef(env, jClassString);
-    }
-#ifdef UNICODE
+TCHAR *JNU_GetNativeWFromString(JNIEnv *env, jstring jstr) {
+    TCHAR* tresult = NULL;
+    const char *result;
 #ifdef WIN32
-    size = MultiByteToWideChar(CP_OEMCP, 0, result, -1, NULL, 0);
-    tresult = malloc(size*sizeof(LPWSTR));
-    if (!tresult) {
-        free(result);
-        throwOutOfMemoryError(env, TEXT("GSNC3"));
-        return NULL;
-    }
-    MultiByteToWideChar(CP_OEMCP, 0, result,-1, tresult, size);
-    free(result);
-    return tresult;
-#else
-    size = mbstowcs(NULL, result, MBSTOWCS_QUERY_LENGTH);
-    if (size == (size_t)-1) {
-        throwJNIError(env, TEXT("Encoding error."));
-        return NULL;
-    }
-    tresult = malloc(sizeof(TCHAR) * (size + 1));
-    if (!tresult) {
-        free(result);
-        throwOutOfMemoryError(env, TEXT("GSNC3"));
-        return NULL;
-    }
-    mbstowcs(tresult, result, size + 1);
-    tresult[size] = TEXT('\0'); /* Avoid bufferflows caused by badly encoded characters. */
-    
-    free(result);
-    return tresult;
+    int size;
 #endif
-#else
-    return result;
-#endif
-}
 
-jstring JNU_NewStringFromNativeChar(JNIEnv *env, const char *str) {
-    jstring result;
-    jclass jClassString;
-    jmethodID MID_String_init;
-    jbyteArray bytes = 0;
-    size_t len;
-
-    result = NULL;
-    if ((*env)->EnsureLocalCapacity(env, 2) < 0) {
-        throwOutOfMemoryError(env, TEXT("JNSN2"));
-        return NULL; /* out of memory error */
-    }
-    
-    len = strlen(str);
-    bytes = (*env)->NewByteArray(env, (jsize)len);
-    if (bytes != NULL) {
-        (*env)->SetByteArrayRegion(env, bytes, 0, (jsize)len,(jbyte*) str);
-        if ((jClassString = (*env)->FindClass(env, utf8ClassJavaLangString)) != NULL) {
-            if ((MID_String_init = (*env)->GetMethodID(env, jClassString, utf8MethodInit, utf8Sig_BrV)) != NULL) {
-                result = (*env)->NewObject(env, jClassString, MID_String_init, bytes);
-            }
-            
-            (*env)->DeleteLocalRef(env, jClassString);
+    result = (*env)->GetStringUTFChars(env, jstr, NULL);
+#ifdef WIN32
+    size = MultiByteToWideChar(CP_UTF8, 0, result, -1, NULL, 0);
+    if (size <= 0) {
+        _tprintf(TEXT("WrapperJNI Warn: Unexpected conversion error: %s\n"), getLastErrorText()); fflush(NULL);
+    } else {
+        tresult = malloc(sizeof(TCHAR) * (size + 1));
+        if (!tresult) {
+            throwOutOfMemoryError(env, TEXT("GSNC1"));
+        } else {
+            MultiByteToWideChar(CP_UTF8, 0, result,-1, tresult, size + 1);
         }
-        
-        (*env)->DeleteLocalRef(env, bytes);
-
-    } /* else fall through */
-    return result;
+    }
+#else
+    if (converterMBToWide(result, MB_UTF8, &tresult, TRUE)) {
+        if (tresult) {
+            _tprintf(tresult); fflush(NULL);
+            free(tresult);
+            tresult = NULL;
+        } else {
+            throwOutOfMemoryError(env, TEXT("GSNC2"));
+        }
+    }
+#endif
+    (*env)->ReleaseStringUTFChars(env, jstr, (const char *)result);
+    return tresult;
 }
 
+#ifdef WIN32
+/* So far this function is only used by windows. if we want to use it for unix as well, first
+   provide correct wchar handling... */
+void JNU_SetByteArrayRegion(JNIEnv *env, jbyteArray* jarray, jsize start, jsize len, const TCHAR *buffer) {
+    char* msg;
+ #if defined(UNICODE) && defined(WIN32)
+    int size;
+
+    size = WideCharToMultiByte(CP_OEMCP, 0, buffer, -1, NULL, 0, NULL, NULL);
+    msg = malloc(size);
+    if (!msg) {
+        throwOutOfMemoryError(env, TEXT("JSBAR1"));
+        return;
+    }
+    WideCharToMultiByte(CP_OEMCP,0, buffer,-1, msg, size, NULL, NULL);
+ #else
+     msg = (TCHAR*) buffer;
+ #endif
+    (*env)->SetByteArrayRegion(env, *jarray, start, len, (jbyte*) msg);
+ #if defined(UNICODE) && defined(WIN32)
+    free(msg);
+ #endif
+}
+#endif
+
 /**
  * Returns a new buffer containing the UTF8 characters for the specified native string.
+ *  Note: this function is ok. Just not sure we need to need to pass through a jstring.
+ *  We could handle the conversion in native code only.
  *
  * It is the responsibility of the caller to free the returned buffer.
  */
@@ -382,7 +354,7 @@ char *getUTF8Chars(JNIEnv *env, const char *nativeChars) {
     jboolean isCopy;
     char *utf8Chars = NULL;
 
-    js = JNU_NewStringFromNativeChar(env, nativeChars);
+    js = JNU_NewStringFromNativeMB(env, nativeChars);
     if (js != NULL) {
         jlen = (*env)->GetStringUTFLength(env, js);
         utf8Chars = malloc(jlen + 1);
@@ -451,11 +423,11 @@ int getSystemProperty(JNIEnv *env, const TCHAR *propertyName, TCHAR **propertyVa
 
     if ((jClassSystem = (*env)->FindClass(env, utf8ClassJavaLangSystem)) != NULL) {
         if ((jMethodIdGetProperty = (*env)->GetStaticMethodID(env, jClassSystem, utf8MethodGetProperty, utf8SigLjavaLangStringrLjavaLangString)) != NULL) {
-            if ((jStringKeyPropName = JNU_NewStringNative(env, propertyName)) != NULL) {
+            if ((jStringKeyPropName = JNU_NewStringFromNativeW(env, propertyName)) != NULL) {
                 if ((jStringKeyValue = (jstring)(*env)->CallStaticObjectMethod(env, jClassSystem, jMethodIdGetProperty, jStringKeyPropName)) != NULL) {
                     /* Collect the value. */
                     if (!encodeNative) {
-                        if ((keyChars = JNU_GetStringNativeChars(env, jStringKeyValue)) != NULL) {
+                        if ((keyChars = JNU_GetNativeWFromString(env, jStringKeyValue)) != NULL) {
                             *propertyValue = malloc(sizeof(TCHAR) * (_tcslen(keyChars) + 1));
                             if (!*propertyValue) {
                                 throwOutOfMemoryError(env, TEXT("GSP1"));
@@ -510,6 +482,90 @@ int getSystemProperty(JNIEnv *env, const TCHAR *propertyName, TCHAR **propertyVa
     return result;
 }
 
+static JavaVM *jvm = NULL;
+static jobject outGlobalRef = NULL;
+static jmethodID printlnMethodId = NULL;
+
+int printMessageCallback(const TCHAR* message) {
+    JNIEnv** envPtr;
+    JNIEnv* env = NULL;
+    jstring jMsg;
+
+    /* Do not print directly to the standard output because the JVM will interpret it with its own encoding (file.encoding)
+     *  which may differ from the locale encoding (especially on Windows or when setting file.encoding from the java additionals).
+     *  Instead we will create a JString and let the WrapperResources print it.
+     *
+     *  Another possibility would be to change the locale so that its encoding matches with the JVM encoding, or to convert the log
+     *  messages to MB using the JVM encoding (we should however use the equivalent code page or iconv syntax). To do this, we would
+     *  need to either include wrapper_jvminfo.c to the native libary (but it would make it heavier), or pass the equivalent encoding
+     *  in a system property (the conversion would be the opposite as when the Wrapper catches JVM outputs). */
+    if (jvm && outGlobalRef && printlnMethodId) {
+        /* envPtr is to avoid a warning "dereferencing type-punned pointer will break strict-aliasing rules" happening on certain platforms. */
+        envPtr = &env;
+        if ((*jvm)->AttachCurrentThread(jvm, (void**)envPtr, NULL) == 0) {
+            if ((jMsg = JNU_NewStringFromNativeW(env, message)) != NULL) {
+                (*env)->CallVoidMethod(env, outGlobalRef, printlnMethodId, jMsg);
+                return FALSE;
+            }
+        }
+    }
+    return TRUE;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeDispose
+ * Signature: (Z)V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeDispose(JNIEnv *env, jclass jClassWrapperManager, jboolean debugging) {
+    if (wrapperJNIDebugging) {
+        /* This is useful for making sure that this is the last JNI call. */
+        log_printf(TEXT("WrapperJNI Debug: Disposing WrapperManager native library."));
+    }
+    (*env)->DeleteGlobalRef(env, outGlobalRef);
+}
+
+/**
+ * Get a pointer to a Java method used to print native messages.
+ *
+ * @return TRUE if there were any problems.
+ */
+int initLog(JNIEnv *env) {
+    jobject outField;
+    jclass systemClass, printStreamClass;
+    jfieldID outFieldId;
+
+    /* Get system class */
+    if ((systemClass = (*env)->FindClass(env, getUTF8Chars(env, "java/lang/System"))) != NULL) {
+        /* Lookup the "out" field */
+        if ((outFieldId = (*env)->GetStaticFieldID(env, systemClass, getUTF8Chars(env, "out"), getUTF8Chars(env, "Ljava/io/PrintStream;"))) != NULL) {
+            /* Get "out" PrintStream instance */
+            if ((outField = (*env)->GetStaticObjectField(env, systemClass, outFieldId)) != NULL) {
+                /* Get PrintStream class */
+                if ((printStreamClass = (*env)->FindClass(env, getUTF8Chars(env, "java/io/PrintStream"))) != NULL) {
+                    /* Lookup println() */
+                    if ((printlnMethodId = (*env)->GetMethodID(env, printStreamClass, getUTF8Chars(env, "println"), getUTF8Chars(env, "(Ljava/lang/String;)V"))) != NULL) {
+                        /* Save a JavaVM* instance to get an environment in our callback method */
+                        if ((*env)->GetJavaVM(env, &jvm) == 0) {
+                            /* Keep a global reference to the out stream for faster reuse. */
+                            if ((outGlobalRef = (*env)->NewGlobalRef(env, outField)) != NULL) {
+                                /* Register a callback method to print our messages */
+                                setPrintMessageCallback(printMessageCallback);
+                                return FALSE;
+                            }
+                        }
+                    }
+                    (*env)->DeleteLocalRef(env, printStreamClass);
+                }
+                (*env)->DeleteLocalRef(env, outField);
+            }
+        }
+        (*env)->DeleteLocalRef(env, systemClass);
+    }
+    return TRUE;
+}
+
 /**
  * Do common initializaion.
  *
@@ -522,6 +578,9 @@ int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
     int errfd;
     int mode;
     int options;
+#ifdef HPUX
+    TCHAR* fixIconvHpuxPropValue = NULL;
+#endif
 
 #ifdef WIN32
     mode = _S_IWRITE;
@@ -532,6 +591,15 @@ int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
 #endif
     initUTF8Strings(env);
 
+#ifdef HPUX
+    if (!getSystemProperty(env, TEXT("wrapper.fix_iconv_hpux"), &fixIconvHpuxPropValue, FALSE)) {
+        if (fixIconvHpuxPropValue && (strcmpIgnoreCase(fixIconvHpuxPropValue, TEXT("ALWAYS")) == 0)) {
+            toggleIconvHpuxFix(TRUE);
+        }
+        free(fixIconvHpuxPropValue);
+    }
+#endif
+
     if (getSystemProperty(env, TEXT("wrapper.java.errfile"), &errfile, FALSE)) {
         /* Failed */
         return TRUE;
@@ -544,25 +612,27 @@ int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
         } else {
             redirectedStdErr = TRUE;
         }
+        free(errfile);
     }
     if (getSystemProperty(env, TEXT("wrapper.java.outfile"), &outfile, FALSE)) {
         /* Failed */
         return TRUE;
     }
     if (outfile) {
-        _tprintf(TEXT("WrapperJNI: Redirecting %s to file %s...\n"), TEXT("StdOut"), outfile); fflush(NULL);
+        log_printf(TEXT("WrapperJNI: Redirecting %s to file %s..."), TEXT("StdOut"), outfile);
         if (((outfd = _topen(outfile, options, mode)) == -1) || (dup2(outfd, STDOUT_FILENO) == -1)) {
             throwThrowable(env, utf8javaIOIOException, TEXT("Failed to redirect %s to file %s  (Err: %s)"), TEXT("StdOut"), outfile, getLastErrorText());
             return TRUE;
         } else {
             redirectedStdOut = TRUE;
         }
+        free(outfile);
     }
-    
+
     return FALSE;
 }
 
-void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt, ...) {
+void throwThrowable(JNIEnv *env, const char *throwableClassName, const TCHAR *lpszFmt, ...) {
     va_list vargs;
     int messageBufferSize = 0;
     TCHAR *messageBuffer = NULL;
@@ -571,6 +641,36 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
     jmethodID constructor;
     jstring jMessageBuffer;
     jobject jThrowable;
+#if defined(UNICODE) && !defined(WIN32)
+    TCHAR *msg = NULL;
+    int i;
+    int flag;
+#endif
+
+#if defined(UNICODE) && !defined(WIN32)
+    if (wcsstr(lpszFmt, TEXT("%s")) != NULL) {
+        msg = malloc(sizeof(wchar_t) * (wcslen(lpszFmt) + 1));
+        if (msg) {
+            /* Loop over the format and convert all '%s' patterns to %S' so the UNICODE displays correctly. */
+            if (wcslen(lpszFmt) > 0) {
+                for (i = 0; i < _tcslen(lpszFmt); i++){
+                    msg[i] = lpszFmt[i];
+                    if ((lpszFmt[i] == TEXT('%')) && (i  < _tcslen(lpszFmt)) && (lpszFmt[i + 1] == TEXT('s')) && ((i == 0) || (lpszFmt[i - 1] != TEXT('%')))){
+                        msg[i+1] = TEXT('S'); i++;
+                    }
+                }
+            }
+            msg[wcslen(lpszFmt)] = TEXT('\0');
+        } else {
+            throwOutOfMemoryError(env, TEXT("TT0"));
+            return;
+        }
+        flag = TRUE;
+    } else {
+        msg = (TCHAR*) lpszFmt;
+        flag = FALSE;
+    }
+#endif
 
     do {
         if (messageBufferSize == 0) {
@@ -583,6 +683,11 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
             messageBuffer = (TCHAR*)malloc( messageBufferSize * sizeof(TCHAR));
             if (!messageBuffer) {
                 throwOutOfMemoryError(env, TEXT("TT1"));
+#if defined(UNICODE) && !defined(WIN32)
+                if (flag == TRUE) {
+                    free(msg);
+                }
+#endif
                 return;
             }
         }
@@ -590,7 +695,11 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
         /* Try writing to the buffer. */
         va_start(vargs, lpszFmt);
 
+#if defined(UNICODE) && !defined(WIN32)
+        count = _vsntprintf(messageBuffer, messageBufferSize, msg, vargs);
+#else
         count = _vsntprintf(messageBuffer, messageBufferSize, lpszFmt, vargs);
+#endif
 
         va_end(vargs);
         if ((count < 0) || (count >= (int)messageBufferSize)) {
@@ -604,9 +713,7 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
             free(messageBuffer);
 
             /* Decide on a new buffer size. */
-            if (count <= (int)messageBufferSize) {
-                messageBufferSize += 50;
-            } else if (count + 1 <= (int)messageBufferSize + 50) {
+            if (count + 1 <= (int)messageBufferSize + 50) {
                 messageBufferSize += 50;
             } else {
                 messageBufferSize = count + 1;
@@ -619,6 +726,11 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
             messageBuffer = (TCHAR*)malloc(messageBufferSize * sizeof(TCHAR));
             if (!messageBuffer) {
                 throwOutOfMemoryError(env, TEXT("TT2"));
+#if defined(UNICODE) && !defined(WIN32)
+                if (flag == TRUE) {
+                    free(msg);
+                }
+#endif
                 return;
             }
 
@@ -627,13 +739,19 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
         }
     } while (count < 0);
 
+#if defined(UNICODE) && !defined(WIN32)
+    if (flag == TRUE) {
+        free(msg);
+    }
+#endif
+
     /* We have the messageBuffer */
     if ((jThrowableClass = (*env)->FindClass(env, throwableClassName)) != NULL) {
         if ((constructor = (*env)->GetMethodID(env, jThrowableClass, utf8MethodInit, utf8SigLjavaLangStringrV)) != NULL) {
-            if ((jMessageBuffer = JNU_NewStringNative(env, messageBuffer)) != NULL) {
+            if ((jMessageBuffer = JNU_NewStringFromNativeW(env, messageBuffer)) != NULL) {
                 if ((jThrowable = (*env)->NewObject(env, jThrowableClass, constructor, jMessageBuffer)) != NULL) {
                     if ((*env)->Throw(env, jThrowable)) {
-                        _tprintf(TEXT("WrapperJNI Error: Unable to throw %s with message: %s"), throwableClassName, messageBuffer); fflush(NULL);
+                        log_printf(TEXT("WrapperJNI Error: Unable to throw %s with message: %s"), throwableClassName, messageBuffer);
                     }
                     (*env)->DeleteLocalRef(env, jThrowable);
                 }
@@ -652,10 +770,9 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
  * @param locationCode The locationCode to help tell where the error happened.
  */
 void throwOutOfMemoryError(JNIEnv *env, const TCHAR* locationCode) {
-    throwThrowable(env, (char*)utf8ClassJavaLangOutOfMemoryError, TEXT("Out of memory (%s)"), locationCode);
+    throwThrowable(env, utf8ClassJavaLangOutOfMemoryError, TEXT("Out of memory (%s)"), locationCode);
 
-    _tprintf(TEXT("WrapperJNI Error: Out of memory (%s)\n"), locationCode);
-    fflush(NULL);
+    log_printf(TEXT("WrapperJNI Error: Out of memory (%s)"), locationCode);
 }
 
 void throwJNIError(JNIEnv *env, const TCHAR *message) {
@@ -667,11 +784,10 @@ void throwJNIError(JNIEnv *env, const TCHAR *message) {
     if ((exceptionClass = (*env)->FindClass(env, utf8ClassOrgTanukisoftwareWrapperWrapperJNIError)) != NULL) {
         /* Look for the constructor. Ignore failures. */
         if ((constructor = (*env)->GetMethodID(env, exceptionClass, utf8MethodInit, utf8Sig_BrV)) != NULL) {
-            if ((jMessage = JNU_NewStringNative(env, message)) != NULL) {
+            if ((jMessage = JNU_NewStringFromNativeW(env, message)) != NULL) {
                 if ((exception = (*env)->NewObject(env, exceptionClass, constructor, jMessage)) != NULL) {
                     if ((*env)->Throw(env, exception)) {
-                        _tprintf(TEXT("WrapperJNI Error: Unable to throw WrapperJNIError with message: %s"), message);
-                        fflush(NULL);
+                        log_printf(TEXT("WrapperJNI Error: Unable to throw WrapperJNIError with message: %s"), message);
                     }
                     (*env)->DeleteLocalRef(env, exception);
                 }
@@ -687,13 +803,11 @@ void throwJNIError(JNIEnv *env, const TCHAR *message) {
 void wrapperJNIHandleSignal(int signal) {
     if (wrapperLockControlEventQueue()) {
         /* Failed.  Should have been reported. */
-        _tprintf(TEXT("WrapperJNI Error: Signal %d trapped, but ignored.\n"), signal);
-        fflush(NULL);
+        log_printf(TEXT("WrapperJNI Error: Signal %d trapped, but ignored."), signal);
         return;
     }
 #ifdef _DEBUG
-    _tprintf(TEXT(" Queue Write 1 R:%d W:%d E:%d\n"), controlEventQueueLastReadIndex, controlEventQueueLastWriteIndex, signal);
-    fflush(NULL);
+    log_printf(TEXT(" Queue Write 1 R:%d W:%d E:%d"), controlEventQueueLastReadIndex, controlEventQueueLastWriteIndex, signal);
 #endif
     controlEventQueueLastWriteIndex++;
     if (controlEventQueueLastWriteIndex >= CONTROL_EVENT_QUEUE_SIZE) {
@@ -701,8 +815,7 @@ void wrapperJNIHandleSignal(int signal) {
     }
     controlEventQueue[controlEventQueueLastWriteIndex] = signal;
 #ifdef _DEBUG
-    _tprintf(TEXT(" Queue Write 2 R:%d W:%d\n"), controlEventQueueLastReadIndex, controlEventQueueLastWriteIndex);
-    fflush(NULL);
+    log_printf(TEXT(" Queue Write 2 R:%d W:%d"), controlEventQueueLastReadIndex, controlEventQueueLastWriteIndex);
 #endif
 
     if (wrapperReleaseControlEventQueue()) {
@@ -720,7 +833,7 @@ void wrapperJNIHandleSignal(int signal) {
 JNIEXPORT jstring JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetLibraryVersion(JNIEnv *env, jclass clazz) {
     jstring version;
-    version = JNU_NewStringNative(env, wrapperVersion);
+    version = JNU_NewStringFromNativeW(env, wrapperVersion);
     return version;
 }
 
@@ -790,14 +903,14 @@ JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_accessViolationInner(JNIEnv *env, jclass clazz) {
     TCHAR *ptr;
 
+    log_printf(TEXT("WrapperJNI Warn: Causing access violation..."));
+    
     /* Cause access violation */
     ptr = NULL;
     ptr[0] = L'\n';
 
 }
 
-
-
 JNIEXPORT jobject JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeExec(JNIEnv *env, jclass jWrapperManagerClass, jobjectArray jCmdArray, jstring jCmdLine, jobject jWrapperProcessConfig, jboolean spawnChDir) {
 
