@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -43,7 +43,7 @@
 #include "loggerjni.h"
 #include "wrapperjni.h"
 
-static pid_t wrapperProcessId = -1;
+pid_t wrapperProcessId = -1;
 pthread_mutex_t controlEventQueueMutex = PTHREAD_MUTEX_INITIALIZER;
 
 
@@ -53,8 +53,7 @@ int wrapperLockControlEventQueue() {
      *  This could happen if a signal is encountered while locked. */
     while (pthread_mutex_trylock(&controlEventQueueMutex) == EBUSY) {
         if (count >= 3000) {
-            _tprintf(TEXT("WrapperJNI Error: Timed out waiting for control event queue lock.\n"));
-            fflush(NULL);
+            log_printf(TEXT("WrapperJNI Error: Timed out waiting for internal lock (%s)."), TEXT("control event queue"));
             return -1;
         }
         wrapperSleep(10);
@@ -64,19 +63,18 @@ int wrapperLockControlEventQueue() {
     if (count > 0) {
         if (wrapperJNIDebugging) {
             /* This is useful for making sure that the JNI call is working. */
-            _tprintf(TEXT("WrapperJNI Debug: wrapperLockControlEventQueue looped %d times before lock.\n"), count);
-            fflush(NULL);
+            log_printf(TEXT("WrapperJNI Debug: Looped %d times before lock (%s)."), count, TEXT("control event queue"));
         }
     }
     return 0;
 }
 
 int wrapperReleaseControlEventQueue() {
-    if (pthread_mutex_unlock(&controlEventQueueMutex)) {
-        _tprintf(TEXT("WrapperJNI Error: Failed to unlock the event queue mutex.\n"));
-        fflush(NULL);
+    int ret = pthread_mutex_unlock(&controlEventQueueMutex);
+    if (ret != 0) {
+        log_printf(TEXT("WrapperJNI Error: Failed to release internal lock (%s, 0x%x)."), TEXT("control event queue"), ret);
     }
-    return 0;
+    return ret;
 }
 
 /**
@@ -138,6 +136,12 @@ JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jClassWrapperManager, jboolean debugging) {
     TCHAR *retLocale;
     wrapperJNIDebugging = debugging;
+#ifdef FREEBSD
+    if (loadIconvLibrary()) {
+        /* Already reported. */
+        return;
+    }
+#endif
 
     /* Set the locale so we can display MultiByte characters. */
     retLocale = _tsetlocale(LC_ALL, TEXT(""));
@@ -146,11 +150,11 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
         free(retLocale);
     }
 #endif
+    initLog(env);
 
     if (wrapperJNIDebugging) {
         /* This is useful for making sure that the JNI call is working. */
-        _tprintf(TEXT("WrapperJNI Debug: Inside native WrapperManager initialization method\n"));
-        fflush(NULL);
+        log_printf(TEXT("WrapperJNI Debug: Inside native WrapperManager initialization method"));
     }
 
     /* Set handlers for signals */
@@ -171,13 +175,42 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
     wrapperProcessId = getpid();
 }
 
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeLoadWrapperProperties
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeLoadWrapperProperties(JNIEnv *env, jclass clazz) {
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeRaiseExceptionInner
+ * Signature: (I)V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseExceptionInner(JNIEnv *env, jclass clazz, jint code) {
+    log_printf(TEXT("WrapperJNI: Not Windows.  RaiseException ignored."));
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeRaiseFailFastExceptionInner
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseFailFastExceptionInner(JNIEnv *env, jclass clazz) {
+    log_printf(TEXT("WrapperJNI: Not Windows.  FailFastException ignored."));
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeRedirectPipes
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *evn, jclass clazz) {
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *env, jclass clazz) {
     int fd;
     
     fd = _topen(TEXT("/dev/null"), O_RDWR, 0);
@@ -192,9 +225,9 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *evn,
         }
         
         if (!redirectedStdOut) {
-            _tprintf(TEXT("WrapperJNI: Redirecting %s to /dev/null\n"), TEXT("StdOut")); fflush(NULL);
+            log_printf(TEXT("WrapperJNI: Redirecting %s to /dev/null"), TEXT("StdOut"));
             if (dup2(fd, STDOUT_FILENO) == -1) {
-                _tprintf(TEXT("WrapperJNI: Failed to redirect %s to /dev/null  (Err: %s)\n"), TEXT("StdOut"), getLastErrorText()); fflush(NULL);
+                log_printf(TEXT("WrapperJNI: Failed to redirect %s to /dev/null  (Err: %s)"), TEXT("StdOut"), getLastErrorText());
             } else {
                 redirectedStdOut = TRUE;
             }
@@ -226,14 +259,12 @@ JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeRequestThreadDump(
         JNIEnv *env, jclass clazz) {
     if (wrapperJNIDebugging) {
-        _tprintf(TEXT("WrapperJNI Debug: Sending SIGQUIT event to process group %d.\n"),
+        log_printf(TEXT("WrapperJNI Debug: Sending SIGQUIT event to process group %d."),
             (int)wrapperProcessId);
-        fflush(NULL);
     }
     if (kill(wrapperProcessId, SIGQUIT) < 0) {
-        _tprintf(TEXT("WrapperJNI Error: Unable to send SIGQUIT to JVM process: %s\n"),
+        log_printf(TEXT("WrapperJNI Error: Unable to send SIGQUIT to JVM process: %s"),
             getLastErrorText());
-        fflush(NULL);
     }
 }
 
@@ -245,8 +276,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRequestThreadDump(
 JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeSetConsoleTitle(JNIEnv *env, jclass clazz, jstring jstringTitle) {
     if (wrapperJNIDebugging) {
-        _tprintf(TEXT("WrapperJNI Debug: Setting the console title not supported on UNIX platforms.\n"));
-        fflush(NULL);
+        log_printf(TEXT("WrapperJNI Debug: Setting the console title not supported on UNIX platforms."));
     }
 }
 
@@ -288,16 +318,16 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetUser(JNIEnv *env, jclass
 
             /* Create the arguments to the constructor as java objects */
             /* User */
-            jstringUser = JNU_NewStringFromNativeChar(env, pw->pw_name);
+            jstringUser = JNU_NewStringFromNativeMB(env, pw->pw_name);
             if (jstringUser) {
                 /* Real Name */
-                jstringRealName = JNU_NewStringFromNativeChar(env, pw->pw_gecos);
+                jstringRealName = JNU_NewStringFromNativeMB(env, pw->pw_gecos);
                 if (jstringRealName) {
                     /* Home */
-                    jstringHome = JNU_NewStringFromNativeChar(env, pw->pw_dir);
+                    jstringHome = JNU_NewStringFromNativeMB(env, pw->pw_dir);
                     if (jstringHome) {
                         /* Shell */
-                        jstringShell = JNU_NewStringFromNativeChar(env, pw->pw_shell);
+                        jstringShell = JNU_NewStringFromNativeMB(env, pw->pw_shell);
                         if (jstringShell) {
                             /* Now create the new wrapperUser using the constructor arguments collected above. */
                             wrapperUser = (*env)->NewObject(env, wrapperUserClass, constructor,
@@ -311,7 +341,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetUser(JNIEnv *env, jclass
                                         ggid = aGroup->gr_gid;
 
                                         /* Group name */
-                                        jstringGroupName = JNU_NewStringFromNativeChar(env, aGroup->gr_name);
+                                        jstringGroupName = JNU_NewStringFromNativeMB(env, aGroup->gr_name);
                                         if (jstringGroupName) {
                                             /* Add the group to the user. */
                                             (*env)->CallVoidMethod(env, wrapperUser, setGroup,
@@ -344,7 +374,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetUser(JNIEnv *env, jclass
                                             ggid = aGroup->gr_gid;
 
                                             /* Group name */
-                                            jstringGroupName = JNU_NewStringFromNativeChar(env, aGroup->gr_name);
+                                            jstringGroupName = JNU_NewStringFromNativeMB(env, aGroup->gr_name);
                                             if (jstringGroupName) {
                                                 /* Add the group to the user. */
                                                 (*env)->CallVoidMethod(env, wrapperUser, addGroup,
@@ -431,4 +461,59 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeSendServiceControlCode(JNIE
     /** Not supported on UNIX platforms. */
     return NULL;
 }
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetPortStatus
+ * Signature: (ILjava/lang/String;I)I
+ *
+ * @param port Port is the port whose status is requested.
+ * @param protocol The protocol of the port, 0=tcpv4, 1=tcpv6
+ *
+ * @return The status, -1=error, 0=closed, >0=in use.
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetPortStatus(JNIEnv *env, jclass clazz, jint port, jstring jAddress, jint protocol) {
+    /* Not implemented on UNIX as it is not needed for now. May implement if ever made publicly available. */
+    return 0;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeSetDpiAwareness
+ * Signature: ()I
+ *
+ * @param awareness The DPI awareness value to set.
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeSetDpiAwareness(JNIEnv *env, jclass clazz, jint awareness) {
+    /* Not implemented on UNIX. */
+    return;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetDpiAwareness
+ * Signature: ()I
+ *
+ * @return The dpi Awareness of the current process, or -1 if there was an error.
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiAwareness(JNIEnv *env, jclass clazz) {
+    /* Not implemented on UNIX. */
+    return -1;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetDpiScale
+ * Signature: ()I
+ *
+ * @return The dpi scale (should be devided by 96 to get the scale factor).
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiScale(JNIEnv *env, jclass clazz) {
+    /* Not implemented on UNIX: always return 96 as it is the default value. */
+    return 96;
+}
 #endif
