@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -13,6 +13,7 @@ package org.tanukisoftware.wrapper;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
@@ -70,7 +71,7 @@ import java.util.jar.Manifest;
  *  state, this means that the Windows Service Manager and several of the
  *  Wrapper's error recovery mechanisms will not function correctly.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class WrapperJarApp
     implements WrapperListener, Runnable
@@ -155,9 +156,32 @@ public class WrapperJarApp
         Class wmClass = WrapperManager.class;
         
         // Set up some log channels
-        m_outInfo = new WrapperPrintStream( System.out, "WrapperJarApp: " );
-        m_outError = new WrapperPrintStream( System.out, "WrapperJarApp Error: " );
-        m_outDebug = new WrapperPrintStream( System.out, "WrapperJarApp Debug: " );
+        boolean streamsSet = false;
+        if ( "true".equals( System.getProperty( "wrapper.use_sun_encoding" ) ) ) {
+            String sunStdoutEncoding = System.getProperty( "sun.stdout.encoding" );
+            if ( ( sunStdoutEncoding != null ) && !sunStdoutEncoding.equals( System.getProperty( "file.encoding" ) ) ) {
+                /* We need to create the stream using the same encoding as the one used for stdout, else this will lead to encoding issues. */
+                try
+                {
+                    m_outInfo = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperJarApp: " );
+                    m_outError = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperJarApp Error: " );
+                    m_outDebug = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperJarApp Debug: " );
+                    streamsSet = true;
+                }
+                catch ( UnsupportedEncodingException e )
+                {
+                    /* This should not happen because we always make sure the encoding exists before launching a JVM.
+                     *  If any of the above streams failed, we want to fall back to streams that use the same encoding. */
+                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding ''{0}'' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                }
+            }
+        }
+        if ( !streamsSet )
+        {
+            m_outInfo = new WrapperPrintStream( System.out, "WrapperJarApp: " );
+            m_outError = new WrapperPrintStream( System.out, "WrapperJarApp Error: " );
+            m_outDebug = new WrapperPrintStream( System.out, "WrapperJarApp Debug: " );
+        }
         
         // Do all of our initialization here so the modified array list which is passed
         //  to the WrapperListener.start method can remain unchanged.  Ideally we would
@@ -171,7 +195,7 @@ public class WrapperJarApp
         if ( args.length < 1 )
         {
             m_initFailed = true;
-            m_initError = WrapperManager.getRes().getString( "Not enough argments.  Minimum {0} required.", "1" );
+            m_initError = WrapperManager.getRes().getString( "Not enough arguments.  Minimum {0} required.", "1" );
             m_initShowUsage = true;
             
             // No jar file name, do the best we can for now.
@@ -510,7 +534,7 @@ public class WrapperJarApp
      * WrapperListener Methods
      *-------------------------------------------------------------*/
     /**
-     * The start method is called when the WrapperManager is signalled by the 
+     * The start method is called when the WrapperManager is signaled by the 
      * native wrapper code that it can start its application.  This
      * method call is expected to return, so a new thread should be launched
      * if necessary.
@@ -688,7 +712,7 @@ public class WrapperJarApp
         System.out.println( WrapperManager.getRes().getString(
             "WrapperJarApp Usage:" ) );
         System.out.println( WrapperManager.getRes().getString(
-            "  java org.tanukisoftware.wrapper.WrapperJarApp {jar_file} [app_arguments]" ) );
+            "  java org.tanukisoftware.wrapper.WrapperJarApp '{'jar_file'}' [app_arguments]" ) );
         System.out.println();
         System.out.println( WrapperManager.getRes().getString(
             "Where:" ) );
