@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -12,6 +12,7 @@ package org.tanukisoftware.wrapper;
  */
 
 import java.io.PrintStream;
+import java.io.UnsupportedEncodingException;
 
 /**
  * Not all methods are currently overridden as this is an internal class.
@@ -21,6 +22,12 @@ final class WrapperPrintStream
 {
     private String m_header;
     
+    /**
+     * Creates a new WrapperPrintStream wrapped around another PrintStream.
+     *
+     * @param out The PrintStream which will be wrapped by this new stream.
+     * @param header The header to append at the beginning of any printed messages.
+     */
     WrapperPrintStream( PrintStream parent, String header )
     {
         super( parent );
@@ -28,6 +35,23 @@ final class WrapperPrintStream
         m_header = header;
     }
     
+    /**
+     * Creates a new WrapperPrintStream wrapped around another PrintStream.
+     *
+     * @param out The PrintStream which will be wrapped by this new stream.
+     * @param autoFlush Whether the output buffer should be automatically flushed or not. The value will be passed to the parent stream.
+     * @param encoding The name of a supported character encoding. The value will be passed to the parent stream.
+     * @param header The header to append at the beginning of any printed messages.
+     *
+     * @throws UnsupportedEncodingException If the named encoding is not supported.
+     */
+    WrapperPrintStream( PrintStream parent, boolean autoFlush, String encoding, String header ) throws UnsupportedEncodingException
+    {
+        super( parent, autoFlush, encoding );
+        
+        m_header = header;
+    }
+    
     /**
      * Terminate the current line by writing the line separator string.  The
      * line separator string is defined by the system property
