@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -19,7 +19,7 @@ import java.io.OutputStream;
  * A WrapperProcess is returned by a call to WrapperManager.exec and can
  *  be used to reference the new child process after it was launched.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  * @since Wrapper 3.4.0
  */
 public class WrapperProcess
@@ -30,6 +30,7 @@ public class WrapperProcess
 
     /* The PID of the process. */
     private int m_pid;
+    private int m_pgid;
     private int m_exitcode;
     private boolean m_isDetached;
     private int m_softShutdownTimeout;
@@ -85,6 +86,16 @@ public class WrapperProcess
         return m_pid;
     }
 
+    /**
+     * Returns the PGID of the process.
+     *
+     * @return The PGID of the process.
+     */
+    public int getPGID()
+    {
+        return m_pgid;
+    }
+
     /**
      * Gets the input stream of the subprocess. The stream obtains data piped
      *  from the standard output stream of the process represented by this
