@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -27,11 +27,11 @@ import org.tanukisoftware.wrapper.WrapperLicenseError;
  *  detailed explanation of how they work.
  * <p>
  * The setter methods are designed to be optionally be chained as follows:
- * <code><pre>
+ * <pre>
  * WrapperProcess proc = WrapperManager.exec( "command", new WrapperProcessConfig().setDetached( true ).setStartType( WrapperProcessConfig.POSIX_SPAWN ) );
- * </pre></code>
+ * </pre>
  *
- * @author Christian Mueller <christian.mueller@tanukisoftware.co.jp>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  * @since Wrapper 3.4.0
  */
 public final class WrapperProcessConfig
@@ -45,6 +45,8 @@ public final class WrapperProcessConfig
     private boolean m_isInteractive;
     private File m_defdir;
     private int m_startType;
+    private boolean m_isNewProcessGroup;
+    private boolean m_isAutoCloseInputStreams;
     private Map m_environment;
     private int m_softShutdownTimeout;
 
@@ -69,8 +71,17 @@ public final class WrapperProcessConfig
         m_isDetached = false;
         m_defdir = null;
         m_startType = DYNAMIC;
+        m_isNewProcessGroup = true;
+        if ( WrapperManager.isWindows() )
+        {
+            m_isAutoCloseInputStreams = false;
+        }
+        else
+        {
+            m_isAutoCloseInputStreams = true;
+        }
         m_environment = null;
-        m_softShutdownTimeout = 5;
+        setSoftShutdownTimeout( 5 );
         m_isInteractive = false;
     }
 
@@ -121,6 +132,10 @@ public final class WrapperProcessConfig
      * Sets the detached flag.  This makes it possible to control whether or
      *  not the Wrapper will terminate any child processes launched by a JVM
      *  when that JVM exits or crashes.
+     *  Note that when running the Wrapper as daemon with systemd, systemd
+     *  may kill all child processes (including detached ones) on shutdown.
+     *  To prevent this, please adjust the value of the SYSTEMD_KILLMODE
+     *  property in the Wrapper Shell script.
      *
      * @param detached If false the Wrapper will remember that the process was
      *                 launched and then make sure that it is terminated when
@@ -187,7 +202,94 @@ public final class WrapperProcessConfig
     {
         verifyStartType( startType );
 
-        m_startType = startType;
+        if ( !WrapperManager.isWindows() )
+        {
+            m_startType = startType;
+        }
+        return this;
+    }
+    
+    /**
+     * Returns the new process group flag.
+     *
+     * @return The new process group flag.
+     */
+    public boolean isNewProcessGroup()
+    {
+        return m_isNewProcessGroup;
+    }
+    
+    /**
+     * Sets the new process group flag.
+     * <p>
+     * When a process is created as a new process group, the Wrapper will monitor and terminate
+     *  all processes in the new process group.
+     * <p>
+     * When false, the process will be a member of the same process group as Java.  The Wrapper
+     *  will only monitor the process which was directly launched.
+     * <p>
+     * Defaults to true.
+     * <p>
+     * On Windows, this value is ignored.  A new process group is required to make sure console
+     *  events are handled correctly.
+     *
+     * @param newProcessGroup True to create a new process group, false to inherit the process group.
+     *
+     * @return This configration to allow chaining.
+     */
+    public WrapperProcessConfig setNewProcessGroup( boolean newProcessGroup )
+    {
+        if ( !WrapperManager.isWindows() )
+        {
+            m_isNewProcessGroup = newProcessGroup;
+        }
+        return this;
+    }
+    
+    /**
+     * Returns whether the input streams of the process should be read in 'auto-close' mode.
+     *
+     * @return true if non-blocking mode is enabled, FALSE otherwise.
+     */
+    public boolean isAutoCloseInputStreams()
+    {
+        return m_isAutoCloseInputStreams;
+    }
+    
+    /**
+     * Specifies how the pipes of the process should be read.
+     * <p>
+     * Two modes are possible:
+     * <p>
+     * With the 'auto-close' mode, which is the default, the input stream will be closed when the process
+     *  ends and there is nothing else to read from the std[out/err] to which it is connected.
+     * <p>
+     * In the other mode, the input stream will wait for data to be read even if the process is gone
+     *  as long as the other end of the stream is open.
+     * <p>
+     * While the second mode might be slightly more efficient during the process of reading data, it can
+     *  cause to block longer after the process termination in case other sub-processes using the same
+     *  standard streams were initiated. These sub-processes may indeed force the standard streams to
+     *  stay open.
+     * <p>
+     * The 'auto-close' mode is also more reliable in the event of the process crashing.
+     * <p>
+     * Defaults to true.
+     * <p>
+     * On z/OS, this value is ignored.  The 'auto-close' mode is required to make sure the pipes are
+     *  read correctly.
+     * On Windows, this value is ignored.  The 'auto-close' mode is not needed.
+     *
+     * @param isAutoCloseInputStreams true to enable 'auto-close' mode, false to disable it.
+     *
+     * @return This configration to allow chaining.
+     */
+    public WrapperProcessConfig setAutoCloseInputStreams( boolean isAutoCloseInputStreams )
+    {
+        if ( !WrapperManager.isWindows() && !WrapperManager.isZOS() )
+        {
+            m_isAutoCloseInputStreams = isAutoCloseInputStreams;
+        }
         return this;
     }
     
@@ -207,9 +309,10 @@ public final class WrapperProcessConfig
      * @param workingDirectory The working directory of the subprocess, or null
      *                         if the subprocess should inherit the working
      *                         directory of the JVM.
-     *                         Please note, when using the POSIX_SPAWN or DYNAMIC start
+     *                         On Unix, when using the POSIX_SPAWN or DYNAMIC start
      *                         type, it is not possible to set the working
      *                         directory.  Doing so will result in an error when running exec.
+     *                         On Windows, it is always possible to set the working directly.
      *
      * @return This configration to allow chaining.
      *
@@ -245,6 +348,9 @@ public final class WrapperProcessConfig
      *  environment being used.
      *  @throws WrapperLicenseError If the function is called other than in
      *                             the Professional Edition or from a Standalone JVM.
+     * 
+     * @return A Map containing the environment which will be used to launch
+     *         the child process.
      */
     public Map getEnvironment()
         throws WrapperLicenseError
@@ -302,6 +408,17 @@ public final class WrapperProcessConfig
 
         return this;
     }
+    
+    
+    /**
+     * Returns the soft shutdown timeout value.
+     *
+     * @return The soft shutdown timeout value.
+     */
+    public int getSoftShutdownTimeout()
+    {
+        return m_softShutdownTimeout;
+    }
 
     /**
      * Sets the timeout for the soft shtudown in seconds.
@@ -315,6 +432,11 @@ public final class WrapperProcessConfig
      * The default value of this property is 5 - giving a process 5 sec to 
      * react on the shutdown request.
      *
+     * On Windows, Window based applications will be asked to shutdown by sending
+     *  WM_CLOSE messages to each of their Window event queues.
+     *  The soft shutdown will be ignored for console processes as it is not
+     *  currently possible to ask them to shutdown cleanly with a CTRL-C.
+     *
      * @param softShutdownTimeout The max timeout for an application to stop, before 
      *                            killing forcibly
      *
@@ -323,7 +445,6 @@ public final class WrapperProcessConfig
      * @throws IllegalArgumentException If the value of the specified timeout is invalid.
      */
     public WrapperProcessConfig setSoftShutdownTimeout( int softShutdownTimeout )
-        throws IOException
     {
         if ( softShutdownTimeout < -1 ) {
             throw new IllegalArgumentException( WrapperManager.getRes().getString( "{0} is not a valid value for a timeout.", 
@@ -413,20 +534,22 @@ public final class WrapperProcessConfig
     }
 
 
-/**
- *  Specifies if the ChildProcesses should be launched
- *  in the current session. 
- *  This property only makes sense if the application was
- *  launched as Windows Service under the System User (or any
- *  other user, having SE_TCB_NAME previledge with the OS)
- *  On non-Windows platforms or when launched in Console mode,
- *  the setting will be ignored silently.
- *
- * @param isInteractive true to enable the feature. 
- */
+    /**
+     *  Specifies if the ChildProcesses should be launched
+     *  in the current session. 
+     *  This property only makes sense if the application was
+     *  launched as Windows Service under the System User (or any
+     *  other user, having SE_TCB_NAME previledge with the OS)
+     *  On non-Windows platforms or when launched in Console mode,
+     *  the setting will be ignored silently.
+     *
+     * @param isInteractive true to enable the feature.
+     * 
+     * @return An instance of WrapperProcessConfig.
+     */
     public WrapperProcessConfig setCreateForActiveUser( boolean isInteractive )
     {
-        if (WrapperManager.isWindows() && WrapperManager.isLaunchedAsService())
+        if ( isInteractive && WrapperManager.isWindows() && WrapperManager.isLaunchedAsService())
         {
             m_isInteractive = true;
         }
@@ -437,9 +560,10 @@ public final class WrapperProcessConfig
         return this;
     }
 
-/**
- *  Tells if the CreateForActiveUser feature was enabled.
- */
+    /**
+     *  Tells if the CreateForActiveUser feature was enabled.
+     * @return CreateForActiveUser.
+     */
     public boolean isCreateForActiveUser()
     {
         return m_isInteractive;
