@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -16,19 +16,40 @@ import java.io.InputStream;
 import java.io.IOException;
 
 /**
- * The InputStream Class of a WrapperProcess, representing all the data the 
+ * The InputStream Class of a WrapperProcess, representing all the data the
  * ChildProcess writes to the Wrapper.
  *
- * @author Christian Mueller <christian.mueller@tanukisoftware.co.jp>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  * @since Wrapper 3.4.0
  */
 public class WrapperProcessInputStream
     extends InputStream
 {
+    /** Platform specific pointer to the underlying native pipe being read from. */
     private long m_ptr;
+    
+    /** Flag which keeps track of whether the close method has been called. */
     private boolean m_closed;
+    
+    /** Buffer containing all unread data from the pipe.  This will only exist if the process terminates before a read method has been called. */
     private ByteArrayInputStream m_bais;
-    private volatile boolean m_read;
+    
+    /** Read flag is set the first time a thread calls any of the read methods.
+     *   This is used to decide whther or not pending data in the buffer needs to be stored in m_bais once the process is terminated.
+     *   Storing the unread data is necessary if the child process terminates before the use code has a chance to access the InputStreams.
+     */
+    private boolean m_read;
+    
+    /** Flag which is set when the child process is terminated.  Used within native code. */
+    private volatile boolean m_terminated;
+    
+    /** Indicates if the pipes should be read in a non-blocking mode with check of the m_terminated flag between retries.  Used within native code. */
+    private volatile boolean m_checkTerminated;
+    
+    /** Semaphore to synchronize access to the m_read flag.
+     *   It is important that the synchronization is made on a distinct object so that m_read can be accessed while other threads hold a lock on 'this'.
+     */
+    private Object m_readSm = new Object();
 
     /*---------------------------------------------------------------
      * Constructors
@@ -45,7 +66,7 @@ public class WrapperProcessInputStream
      *-------------------------------------------------------------*/
     private native int nativeRead( boolean blocking );
     private native void nativeClose();
-    private native int nativeRead2( byte[] b, int off, int len, boolean blocking );
+    private native int nativeRead2( byte[] b, int off, int len );
 
     /*---------------------------------------------------------------
      * Methods
@@ -85,7 +106,7 @@ public class WrapperProcessInputStream
     }
 
     /**
-     *
+     * @return TRUE if ready, otherwise FALSE.
      */
     public boolean ready()
     {
@@ -114,9 +135,16 @@ public class WrapperProcessInputStream
     public int read()
         throws IOException
     {
+        if ( !m_read )
+        {
+            synchronized ( m_readSm )
+            {
+                m_read = true;
+            }
+        }
+
         synchronized( this )
         {
-            m_read = true;
             if ( ( !m_closed ) && WrapperManager.isNativeLibraryOk() )
             {
                 return nativeRead( true );
@@ -206,8 +234,16 @@ public class WrapperProcessInputStream
     public int read( byte b[], int off, int len )
         throws IOException
     {
-        int c;
-        synchronized (this) {
+        if ( !m_read )
+        {
+            synchronized ( m_readSm )
+            {
+                m_read = true;
+            }
+        }
+
+        synchronized ( this )
+        {
             if ( b == null )
             {
                 throw new NullPointerException();
@@ -224,29 +260,23 @@ public class WrapperProcessInputStream
             {
                 return -1;
             }
-            m_read = true;
+            int result;
             if ( ( !m_closed ) && WrapperManager.isNativeLibraryOk() )
             {
-                // Attempt to read output in blocking mode.
-                c = nativeRead2( b, off, len, true );
-                if ( c == -1 ) // a process can terminate only once
-                {
-                    // And end of file was encountered.  This can happen
-                    c = nativeRead2( b, off, len, false );
-                }
+                result = nativeRead2( b, off, len );
             }
             else
             {
                 if ( m_bais != null )
                 {
-                    c = m_bais.read( b, off, len );
+                    result = m_bais.read( b, off, len );
                 }
                 else
                 {
                     throw new IOException(WrapperManager.getRes().getString( "Stream is closed." ) );
                 }
             }
-            return c == 0 ? -1 : c;
+            return result;
         }
     }
 
@@ -261,10 +291,17 @@ public class WrapperProcessInputStream
      */
     private void readAndCloseOpenFDs()
     {
-        if ( m_read )
+        // Set the terminated flag.  This must be done even when read has already been called.
+        //  It is also important that this be set even if another thread is currently reading.
+        m_terminated = true;
+        
+        synchronized ( m_readSm )
         {
-            // Another thread is reading from the stream so we can trust that thread to complete the reads and close on its own.
-            return;
+            if ( m_read )
+            {
+                // Another thread is reading from the stream so we can trust that thread to complete the reads and close on its own.
+                return;
+            }
         }
 
         synchronized( this )
@@ -300,4 +337,3 @@ public class WrapperProcessInputStream
         }
     }
 }
-
