@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -18,7 +18,7 @@ package org.tanukisoftware.wrapper;
  * System Properties can be accessed with the WrapperSystemPropertyUtil
  *  class.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public final class WrapperPropertyUtil
 {
@@ -55,12 +55,19 @@ public final class WrapperPropertyUtil
     public static boolean getBooleanProperty( String name, boolean defaultValue )
     {
         String val = getStringProperty( name, null );
-        if ( val == null )
+        if ( val != null )
         {
-            return defaultValue;
+            if ( val.equalsIgnoreCase( "TRUE" ) )
+            {
+                return true;
+            }
+            else if ( val.equalsIgnoreCase( "FALSE" ) )
+            {
+                return false;
+            }
         }
         
-        return val.equalsIgnoreCase( "TRUE" );
+        return defaultValue;
     }
     
     /**
