@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -29,6 +29,7 @@ package org.tanukisoftware.wrapper;
  * included in all copies or substantial portions of the Software.
  */
 
+import java.io.UnsupportedEncodingException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
@@ -79,7 +80,7 @@ import java.lang.reflect.Modifier;
  *  state, this means that the Windows Service Manager and several of the
  *  Wrapper's error recovery mechanisms will not function correctly.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class WrapperSimpleApp
     implements WrapperListener, Runnable
@@ -166,9 +167,32 @@ public class WrapperSimpleApp
         m_mainMethod = null;
         
         // Set up some log channels
-        m_outInfo = new WrapperPrintStream( System.out, "WrapperSimpleApp: " );
-        m_outError = new WrapperPrintStream( System.out, "WrapperSimpleApp Error: " );
-        m_outDebug = new WrapperPrintStream( System.out, "WrapperSimpleApp Debug: " );
+        boolean streamsSet = false;
+        if ( "true".equals( System.getProperty( "wrapper.use_sun_encoding" ) ) ) {
+            String sunStdoutEncoding = System.getProperty( "sun.stdout.encoding" );
+            if ( ( sunStdoutEncoding != null ) && !sunStdoutEncoding.equals( System.getProperty( "file.encoding" ) ) ) {
+                /* We need to create the stream using the same encoding as the one used for stdout, else this will lead to encoding issues. */
+                try
+                {
+                    m_outInfo = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp: " );
+                    m_outError = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp Error: " );
+                    m_outDebug = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp Debug: " );
+                    streamsSet = true;
+                }
+                catch ( UnsupportedEncodingException e )
+                {
+                    /* This should not happen because we always make sure the encoding exists before launching a JVM.
+                     *  If any of the above streams failed, we want to fall back to streams that use the same encoding. */
+                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding ''{0}'' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                }
+            }
+        }
+        if ( !streamsSet )
+        {
+            m_outInfo = new WrapperPrintStream( System.out, "WrapperSimpleApp: " );
+            m_outError = new WrapperPrintStream( System.out, "WrapperSimpleApp Error: " );
+            m_outDebug = new WrapperPrintStream( System.out, "WrapperSimpleApp Debug: " );
+        }
         
         // Do all of our initialization here so the modified array list which is passed
         //  to the WrapperListener.start method can remain unchanged.  Ideally we would
@@ -182,7 +206,7 @@ public class WrapperSimpleApp
         if ( args.length < 1 )
         {
             m_initFailed = true;
-            m_initError = WrapperManager.getRes().getString( "Not enough argments.  Minimum {0} required.", "1" );
+            m_initError = WrapperManager.getRes().getString( "Not enough arguments.  Minimum {0} required.", "1" );
             m_initShowUsage = true;
             
             // No main class, do the best we can for now.
@@ -413,7 +437,7 @@ public class WrapperSimpleApp
      * WrapperListener Methods
      *-------------------------------------------------------------*/
     /**
-     * The start method is called when the WrapperManager is signalled by the 
+     * The start method is called when the WrapperManager is signaled by the 
      * native wrapper code that it can start its application.  This
      * method call is expected to return, so a new thread should be launched
      * if necessary.
@@ -595,7 +619,7 @@ public class WrapperSimpleApp
         System.out.println( WrapperManager.getRes().getString(
             "WrapperSimpleApp Usage:" ) );
         System.out.println( WrapperManager.getRes().getString(
-            "  java org.tanukisoftware.wrapper.WrapperSimpleApp {app_class{/app_method}} [app_arguments]" ) );
+            "  java org.tanukisoftware.wrapper.WrapperSimpleApp '{'app_class'{'/app_method'}}' [app_arguments]" ) );
         System.out.println();
         System.out.println( WrapperManager.getRes().getString(
             "Where:" ) );
