@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.demo;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -13,6 +13,7 @@ package org.tanukisoftware.wrapper.demo;
 
 import java.awt.Color;
 import java.io.BufferedReader;
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.PrintStream;
@@ -41,7 +42,7 @@ import org.tanukisoftware.wrapper.WrapperSystemPropertyUtil;
  * <a href="http://wrapper.tanukisoftware.com/doc/english/integrate.html">
  * integration</a> section of the documentation for more details.
  * 
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class DemoApp implements WrapperListener
 {
@@ -53,7 +54,10 @@ public class DemoApp implements WrapperListener
     private static Process m_testCase;
     private static PrintStream m_childPrintStream;
     private static WrapperResources m_res;
+    private File m_wrapperBin;
+    private String m_wrapperBinS;
     private String m_confFile;
+    private String m_loggingProps;
 
     /*---------------------------------------------------------------
      * Constructors
@@ -61,8 +65,43 @@ public class DemoApp implements WrapperListener
     private DemoApp()
     {
         m_isTestCaseRunning = false;
+        
+        File wrapperBinDeltaPack;
+        File wrapperBinStd;
+        // See if the binary exists with delta pack naming.
+        if ( WrapperManager.isWindows() )
+        {
+            wrapperBinDeltaPack = new File( "../bin/" + WrapperManager.generateDetailedNativeBaseName( "wrapper" ) + ".exe" );
+            wrapperBinStd = new File( "../bin/wrapper.exe" );
+        }
+        else
+        {
+            wrapperBinDeltaPack = new File( "../bin/" + WrapperManager.generateDetailedNativeBaseName( "wrapper" ) );
+            wrapperBinStd = new File( "../bin/wrapper" );
+        }
+        if ( wrapperBinDeltaPack.exists() )
+        {
+            m_wrapperBin = wrapperBinDeltaPack;
+        }
+        else if ( wrapperBinStd.exists() )
+        {
+            m_wrapperBin = wrapperBinStd;
+        }
+        else
+        {
+            throw new IllegalStateException( getRes().getString( "Unable to locate Wrapper binary at {0} or {1}", wrapperBinDeltaPack.toString(), wrapperBinStd.toString() ) );
+        }
+        
+        m_wrapperBinS = m_wrapperBin.toString();
+        if ( WrapperManager.isWindows() )
+        {
+            m_wrapperBinS = m_wrapperBinS.replace( '/', '\\' );
+        }
+        System.out.println( "wrapperBinS=" + m_wrapperBinS );
+        
         m_confFile = "../conf/demoapp.conf";
-
+        
+        m_loggingProps = "wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.logfile=";
     }
 
     protected DemoAppMainFrame getFrame()
@@ -211,8 +250,12 @@ public class DemoApp implements WrapperListener
                         try
                         {
                             command = br.readLine();
+                            if ( command == null )
+                            {
+                                command = "finish";
+                            }
+                            
                             System.out.println( getRes().getString( "Read action: {0}", command ) );
-        
                             if ( command.equals( "crash" ) )
                             {
                                 Thread.sleep( 5000 );
@@ -416,16 +459,8 @@ public class DemoApp implements WrapperListener
                     // TODO Auto-generated catch block
                     e.printStackTrace();
                 }
-                String wrapperBin;
-                if ( WrapperManager.isWindows() )
-                {
-                    wrapperBin = "../bin/wrapper.exe";
-                }
-                else
-                {
-                    wrapperBin = "../bin/wrapper";
-                }
-                String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=start " + md.getEvents()
+                
+                String arg = m_wrapperBinS + " -c " + m_confFile + " " + m_loggingProps + " wrapper.app.parameter.1=start " + md.getEvents()
                         + " wrapper.event.default.email.debug=TRUE wrapper.event.default.email.smtp.host=" + md.getServer() + " wrapper.event.default.email.smtp.port="
                         + md.getPort() + " wrapper.event.default.email.sender=" + md.getSender() + " wrapper.event.default.email.recipient=" + md.getRecipients();
                 // System.out.println( "execing: " + arg );
@@ -495,7 +530,7 @@ public class DemoApp implements WrapperListener
         }
         else if ( action.equals( "service" ) )
         {
-            p = Runtime.getRuntime().exec( "..\\bin\\wrapper.exe -it " + m_confFile + " wrapper.console.flush=TRUE" );
+            p = Runtime.getRuntime().exec( m_wrapperBinS + " -it " + m_confFile + " " + m_loggingProps );
             // WrapperManager.exec( "sudo ../test/demoapp remove" );
             if ( p != null )
             {
@@ -544,16 +579,7 @@ public class DemoApp implements WrapperListener
         }
         else if ( action.equals( "start" ) )
         {
-            String wrapperBin;
-            if ( WrapperManager.isWindows() )
-            {
-                wrapperBin = "../bin/wrapper.exe";
-            }
-            else
-            {
-                wrapperBin = "../bin/wrapper";
-            }
-            String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=" + action;
+            String arg = m_wrapperBinS + " -c " + m_confFile + " " + m_loggingProps + " wrapper.app.parameter.1=" + action;
             // System.out.println( "calling: " + arg );
 
             if ( !m_isTestCaseRunning )
@@ -591,28 +617,19 @@ public class DemoApp implements WrapperListener
             String input = "";
             if ( System.getProperty( "os.name" ).indexOf( "Windows" ) >= 0 )
             {
-                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the Command, you wish to execute:" ),
+                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the command you wish to execute:" ),
                           getRes().getString( "Child Process Execution" ), JOptionPane.QUESTION_MESSAGE, null, null,
                           ( Object )"notepad" );
             }
             else
             {
-                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the Command, you wish to execute:" ),
+                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the command you wish to execute:" ),
                           getRes().getString( "Child Process Execution" ), JOptionPane.QUESTION_MESSAGE, null, null,
                           ( Object )"xclock" );
             }
             if ( input != null && input.length() > 0 )
             {
-                String wrapperBin;
-                if ( WrapperManager.isWindows() )
-                {
-                    wrapperBin = "../bin/wrapper.exe";
-                }
-                else
-                {
-                    wrapperBin = "../bin/wrapper";
-                }
-                String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=start";
+                String arg = m_wrapperBinS + " -c " + m_confFile + " " + m_loggingProps + " wrapper.app.parameter.1=start";
                 //System.out.println( "calling: " + arg );
                 if ( !m_isTestCaseRunning )
                 {
@@ -641,16 +658,7 @@ public class DemoApp implements WrapperListener
         }
         else
         {
-            String wrapperBin;
-            if ( WrapperManager.isWindows() )
-            {
-                wrapperBin = "../bin/wrapper.exe";
-            }
-            else
-            {
-                wrapperBin = "../bin/wrapper";
-            }
-            String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=start";
+            String arg = m_wrapperBinS + " -c " + m_confFile + " " + m_loggingProps + " wrapper.app.parameter.1=start";
             //System.out.println( "calling: " + arg );
             if ( !m_isTestCaseRunning )
             {
@@ -716,6 +724,7 @@ public class DemoApp implements WrapperListener
      * IMPORTANT: Please read the Javadocs for this class at the top of the page
      * before you start to use this class as a template for integrating your own
      * application. This will save you a lot of time.
+     * @param args Main arguments.
      */
     public static void main( String[] args )
     {
