@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.demo;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -111,7 +111,7 @@ public class DemoAppMainFrame extends JFrame implements ActionListener, WindowLi
         jMenu1.add( jMenuItem2 );
 
         JMenuItem jMenuItem3 = new JMenuItem();
-        jMenuItem3.setAccelerator( javax.swing.KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_F4, java.awt.event.InputEvent.ALT_MASK ) );
+        jMenuItem3.setAccelerator( javax.swing.KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_F4, java.awt.event.InputEvent.ALT_DOWN_MASK ) );
         jMenuItem3.setText( DemoApp.getRes().getString( "Close" ) );
         jMenuItem3.addActionListener( new ActionListener()
         {
@@ -148,7 +148,7 @@ public class DemoAppMainFrame extends JFrame implements ActionListener, WindowLi
         tabbedPane.addTab( DemoApp.getRes().getString( "Failure Detections" ), panel1 );
         tabbedPane.setMnemonicAt( 0, KeyEvent.VK_1 );
 
-        buildCommand( panel1, gridBag1, c1, 1, DemoApp.getRes().getString( "Crash" ), "crash", DemoApp.getRes().getString( "Simulate a Application Crash" ) );
+        buildCommand( panel1, gridBag1, c1, 1, DemoApp.getRes().getString( "Crash" ), "crash", DemoApp.getRes().getString( "Simulate an Application Crash" ) );
         buildCommand( panel1, gridBag1, c1, 1, DemoApp.getRes().getString( "Out of Memory" ), "out_of_mem", DemoApp.getRes().getString( "Simulate a Out Of Memory Error" ) );
         buildCommand( panel1, gridBag1, c1, 1, DemoApp.getRes().getString( "Frozen" ), "frozen", DemoApp.getRes().getString( "Simulate a Frozen JVM" ) );
         buildCommand( panel1, gridBag1, c1, 2, DemoApp.getRes().getString( "Deadlock" ), "deadlock", DemoApp.getRes().getString( "Simulate a Thread Deadlock" ) );
