@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.event;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -21,7 +21,7 @@ import org.tanukisoftware.wrapper.WrapperManager;
  * See the wrapper.pausable and wrapper.pausable.stop_jvm properties for more
  *  information.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  *
  * @since Wrapper 3.5.0
  */
@@ -41,7 +41,7 @@ public abstract class WrapperServiceActionEvent
     
     /**
      * Action result of a configured filter being fired.
-     *  See the wrapper.filter.action.<n> property.
+     *  See the wrapper.filter.action.&lt;n&gt; property.
      */
     public static final int SOURCE_CODE_FILTER                  = 1;
     
@@ -60,25 +60,31 @@ public abstract class WrapperServiceActionEvent
     
     /**
      * Action result of a matched exit code.
-     *  See the wrapper.on_exit.<n> property.
+     *  See the wrapper.on_exit.&lt;n&gt; property.
      */
-    public static final int SOURCE_CODE_ON_EXIT             = 4;
+    public static final int SOURCE_CODE_ON_EXIT                 = 4;
+    
+    /**
+     * Action result of a signal.
+     *  See the wrapper.on_exit.&lt;n&gt; property.
+     */
+    public static final int SOURCE_CODE_SIGNAL                  = 5;
     
     /**
      * Action result of a deadlock being detected.
      *  See the wrapper.check.deadlock.action property.
      */
-    public static final int SOURCE_CODE_DEADLOCK             = 10;
+    public static final int SOURCE_CODE_DEADLOCK                = 10;
     
     /**
      * Action result of a configured timer being fired.
-     *  See the wrapper.timer.<n>.action property.
+     *  See the wrapper.timer.&lt;n&gt;.action property.
      */
     public static final int SOURCE_CODE_TIMER                   = 21;
     
     /**
      * Action result of an event command's block timeout expired.
-     *  See the wrapper.event.<event_name>.command.block.action property.
+     *  See the wrapper.event.&lt;event_name&gt;.command.block.action property.
      */
     public static final int SOURCE_CODE_COMMAND_BLOCK_TIMEOUT   = 22;
     
@@ -113,6 +119,9 @@ public abstract class WrapperServiceActionEvent
         case SOURCE_CODE_ON_EXIT:
             return WrapperManager.getRes().getString( "On Exit Action" );
             
+        case SOURCE_CODE_SIGNAL:
+            return WrapperManager.getRes().getString( "Signal Action" );
+            
         case SOURCE_CODE_DEADLOCK:
             return WrapperManager.getRes().getString( "Deadlock Action" );
             
