@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.security;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -38,7 +38,7 @@ import org.tanukisoftware.wrapper.WrapperManager;
  * </pre>
  * <p>
  * Possible eventTypes include the following:
- * <table border='1' cellpadding='2' cellspacing='0'>
+ * <table border='1' cellpadding='2' cellspacing='0' summary='Possible eventTypes'>
  *   <tr>
  *     <th>Permission Event Type Name</th>
  *     <th>What the Permission Allows</th>
@@ -72,7 +72,7 @@ import org.tanukisoftware.wrapper.WrapperManager;
  *   </tr>
  * </table>
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class WrapperEventPermission
     extends Permission
@@ -84,12 +84,14 @@ public class WrapperEventPermission
     
     public static String EVENT_TYPE_SERVICE = "service";
     public static String EVENT_TYPE_CONTROL = "control";
+    public static String EVENT_TYPE_REMOTE_CONTROL = "remoteControl";
     public static String EVENT_TYPE_CORE = "core";
     
     private static int MASK_SERVICE = 1;
     private static int MASK_CONTROL = 2;
+    private static int MASK_REMOTE_CONTROL = 8;
     private static int MASK_CORE = 65536;
-    private static int MASK_ALL = MASK_SERVICE | MASK_CONTROL | MASK_CORE;
+    private static int MASK_ALL = MASK_SERVICE | MASK_CONTROL | MASK_REMOTE_CONTROL | MASK_CORE;
     
     private int m_eventTypeMask;
     
@@ -174,6 +176,18 @@ public class WrapperEventPermission
             }
             sb.append( EVENT_TYPE_CONTROL );
         }
+        if ( ( m_eventTypeMask & MASK_REMOTE_CONTROL ) != 0 )
+        {
+            if ( first )
+            {
+                sb.append( ',' );
+            }
+            else
+            {
+                first = false;
+            }
+            sb.append( EVENT_TYPE_REMOTE_CONTROL );
+        }
         if ( ( m_eventTypeMask & MASK_CORE ) != 0 )
         {
             if ( first )
@@ -193,12 +207,12 @@ public class WrapperEventPermission
     /**
      * Checks if this WrapperEventPermission object "implies" the
      *  specified permission.
-     * <P>
-     * More specifically, this method returns true if:<p>
+     * <p>
+     * More specifically, this method returns true if:
      * <ul>
-     *  <li><i>p2</i> is an instanceof FilePermission,<p>
+     *  <li><i>p2</i> is an instanceof FilePermission,
      *  <li><i>p2</i>'s eventTypes are a proper subset of this object's eventTypes,
-     *      and<p>
+     *      and
      *  <li><i>p2</i>'s service name is implied by this object's service name.
      *      For example, "MyApp*" implies "MyApp".
      * </ul>
@@ -297,6 +311,10 @@ public class WrapperEventPermission
         {
             return MASK_CONTROL;
         }
+        else if ( eventTypes == EVENT_TYPE_REMOTE_CONTROL )
+        {
+            return MASK_REMOTE_CONTROL;
+        }
         else if ( eventTypes == EVENT_TYPE_CORE )
         {
             return MASK_CORE;
@@ -319,6 +337,10 @@ public class WrapperEventPermission
             {
                 mask |= MASK_CONTROL;
             }
+            else if ( eventType.equals( EVENT_TYPE_REMOTE_CONTROL ) )
+            {
+                mask |= MASK_REMOTE_CONTROL;
+            }
             else if ( eventType.equals( EVENT_TYPE_CORE ) )
             {
                 mask |= MASK_CORE;
