@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -33,8 +33,11 @@ import java.io.BufferedReader;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.util.Enumeration;
 import java.util.Properties;
+import java.text.SimpleDateFormat;
+import java.util.Date;
 
 import org.tanukisoftware.wrapper.WrapperManager;
 import org.tanukisoftware.wrapper.WrapperProcess;
@@ -46,7 +49,7 @@ import org.tanukisoftware.wrapper.event.WrapperEvent;
 import org.tanukisoftware.wrapper.event.WrapperEventListener;
 
 /**
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public abstract class AbstractActionApp
     implements WrapperEventListener
@@ -67,6 +70,7 @@ public abstract class AbstractActionApp
     
     private long m_eventMask = 0xffffffffffffffffL;
     private int m_slowSeconds = 0;
+    private int m_suspendSeconds = 0;
     private String m_serviceName = "testWrapper";
     private String m_consoleTitle = "Java Service Wrapper";
     private String m_childCommand = "ls";
@@ -95,7 +99,7 @@ public abstract class AbstractActionApp
      * Constructors
      *-------------------------------------------------------------*/
     protected AbstractActionApp() {
-        m_runner = new Thread( Main.getRes().getString( "WrapperActionTest_Runner" ) )
+        m_runner = new Thread( "WrapperActionTest_Runner" )
         {
             public void run()
             {
@@ -173,6 +177,11 @@ public abstract class AbstractActionApp
         m_slowSeconds = slowSeconds;
     }
     
+    protected void setSuspendSeconds( int suspendSeconds )
+    {
+        m_suspendSeconds = suspendSeconds;
+    }
+    
     protected void setServiceName( String serviceName )
     {
         m_serviceName = serviceName;
@@ -191,11 +200,42 @@ public abstract class AbstractActionApp
     
     protected void prepareSystemOutErr()
     {
-        m_out = new DeadlockPrintStream( System.out );
+        boolean streamsSet = false;
+        if ( "true".equals( System.getProperty( "wrapper.use_sun_encoding" ) ) ) {
+            String sunStdoutEncoding = System.getProperty( "sun.stdout.encoding" );
+            if ( ( sunStdoutEncoding != null ) && !sunStdoutEncoding.equals( System.getProperty( "file.encoding" ) ) ) {
+                /* We need to create the stream using the same encoding as the one used for stdout, else this will lead to encoding issues. */
+                try
+                {
+                    m_out = new DeadlockPrintStream( System.out, false, sunStdoutEncoding );
+                    m_err = new DeadlockPrintStream( System.err, false, sunStdoutEncoding );
+                    streamsSet = true;
+                }
+                catch ( UnsupportedEncodingException e )
+                {
+                    /* This should not happen because we always make sure the encoding exists before launching a JVM.
+                     *  If any of the above streams failed, we want to fall back to streams that use the same encoding.
+                     *  This message can be localized because we did not redirect the streams yet. */
+                    System.out.println( Main.getRes().getString( "Failed to set the encoding ''{0}'' when creating a DeadlockPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                }
+            }
+        }
+        if ( !streamsSet )
+        {
+            m_out = new DeadlockPrintStream( System.out );
+            m_err = new DeadlockPrintStream( System.err );
+        }
+        
         System.setOut( m_out );
-        m_err = new DeadlockPrintStream( System.err );
         System.setErr( m_err );
     }
+    
+    protected void updateEventListener()
+    {
+        WrapperManager.removeWrapperEventListener( this );
+        WrapperManager.addWrapperEventListener( this, m_eventMask );
+    }
+    
     protected boolean doAction( String action )
     {
         if ( action.equals( "stop0" ) )
@@ -264,6 +304,16 @@ public abstract class AbstractActionApp
         {
             WrapperManager.accessViolationNative();
             
+        }
+        else if ( action.equals( "exception_native" ) )
+        {
+            WrapperManager.raiseExceptionNative( 0x12345678 );
+            
+        }
+        else if ( action.equals( "ff_exception_native" ) )
+        {
+            WrapperManager.raiseFailFastExceptionNative();
+            
         }
         else if ( action.equals( "appear_hung" ) )
         {
@@ -274,6 +324,21 @@ public abstract class AbstractActionApp
         {
             WrapperManager.appearSlow( m_slowSeconds );
             
+        }
+        else if ( action.equals( "appear_slow_1" ) )
+        {
+            WrapperManager.appearSlow( 1 );
+            
+        }
+        else if ( action.equals( "appear_slow_5" ) )
+        {
+            WrapperManager.appearSlow( 5 );
+            
+        }
+        else if ( action.equals( "appear_slow_reset" ) )
+        {
+            WrapperManager.appearSlow( 0 );
+            
         }
         else if ( action.equals( "deadlock" ) )
         {
@@ -302,11 +367,55 @@ public abstract class AbstractActionApp
         }
         else if ( action.equals( "deadlock_out" ) )
         {
-            System.out.println( Main.getRes().getString( "Deadlocking System.out and System.err ..." ) );
+            System.out.println( Main.getRes().getString( "Deadlocking System.out and System.err..." ) );
             m_out.setDeadlock( true );
             m_err.setDeadlock( true );
             
         }
+        else if ( action.equals( "suspend_timeouts" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.suspendTimeouts( m_suspendSeconds );
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
+        else if ( action.equals( "suspend_timeouts_10" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.suspendTimeouts( 10 );
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
+        else if ( action.equals( "suspend_timeouts_60" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.suspendTimeouts( 60 );
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
+        else if ( action.equals( "resume_timeouts" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.resumeTimeouts();
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
         else if ( action.equals( "users" ) )
         {
             if ( !m_users )
@@ -357,7 +466,6 @@ public abstract class AbstractActionApp
                 {
                     public void run()
                     {
-                        System.out.println();
                         System.out.println( Main.getRes().getString( "Start prompting for actions." ) );
                         try
                         {
@@ -365,25 +473,55 @@ public abstract class AbstractActionApp
                             String line;
                             try
                             {
-                                do {
-                                    System.out.println( Main.getRes().getString( "Input an action ('help' for a list of actions):") );
-                                    line = r.readLine();
-                                    if ((line != null) && (!line.equals(""))) {
-                                        System.out.println(Main.getRes().getString( "Read action: {0}", line ) );
-                                        if ( !doAction( line ) )
+                                do
+                                {
+                                    System.out.println( Main.getRes().getString( "Input an action (''help'' for a list of actions):") );
+                                    
+                                    int count = 0;
+                                    int maxTries = 10;
+                                    while( true )
+                                    {
+                                        /* On some old JVMs, we may get an "Interrupted system call" exception if a child process is writing some output while tying to read from here.
+                                         *  It would most likely work if we try again though. */
+                                        try
                                         {
-                                            if ( !line.equals( "help" ) )
+                                            line = r.readLine();
+                                            break;
+                                        }
+                                        catch ( IOException e )
+                                        {
+                                            if ( ( e.toString().indexOf( "Interrupted system call" ) == -1 ) || ( ++count == maxTries ) )
                                             {
-                                                System.out.println( Main.getRes().getString( "Unknown action: {0}", line ) );
+                                                throw e;
                                             }
-                                            printActions();
+                                        }
+                                    }
+                                    if ( line != null )
+                                    {
+                                        if ( !line.equals("") )
+                                        {
+                                            System.out.println(Main.getRes().getString( "Read action: {0}", line ) );
+                                            if ( !doAction( line ) )
+                                            {
+                                                if ( !line.equals( "help" ) )
+                                                {
+                                                    System.out.println( Main.getRes().getString( "Unknown action: {0}", line ) );
+                                                }
+                                                printActions();
+                                            }
+                                        }
+                                        else
+                                        {
+                                            System.out.println(Main.getRes().getString( "Read action: <EMPTY>" ) );
                                         }
                                     }
                                     else
                                     {
-                                        System.out.println(Main.getRes().getString( "Read action: <EMPTY>" ) );
+                                        // EOF
+                                        System.out.println(Main.getRes().getString( "Stdin was closed." ) );
+                                        break;
                                     }
-                                } while (true);
+                                } while ( true );
                             }
                             catch ( IOException e )
                             {
@@ -438,8 +576,7 @@ public abstract class AbstractActionApp
         else if ( action.equals( "listener" ) )
         {
             System.out.println( Main.getRes().getString( "Updating Event Listeners:" ) );
-            WrapperManager.removeWrapperEventListener( this );
-            WrapperManager.addWrapperEventListener( this, m_eventMask );
+            updateEventListener();
         }
         else if ( action.equals( "service_list" ) )
         {
@@ -554,6 +691,11 @@ public abstract class AbstractActionApp
         {
             System.out.println( Main.getRes().getString( "Standard Edition: " ) + WrapperManager.isStandardEdition() );
         }
+        else if ( action.equals( "current_time" ) )
+        {
+            SimpleDateFormat df = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSS" );
+            System.out.println( df.format( new Date() ) + " - " + System.getProperty( "user.timezone" ) );
+        }
         else if ( action.startsWith( "exec " ) && ( action.length() > 5 )  )
         {
             String command = action.substring( 5 );
@@ -624,25 +766,25 @@ public abstract class AbstractActionApp
             {
                 processConfig.setStartType( WrapperProcessConfig.FORK_EXEC );
                 type = "FORK_EXEC";
-                command.substring( 10 );
+                command = command.substring( 10 );
             }
             else if ( command.startsWith( "POSIX_SPAWN " ) )
             {
                 processConfig.setStartType( WrapperProcessConfig.POSIX_SPAWN );
                 type = "POSIX_SPAWN";
-                command.substring( 12 );
+                command = command.substring( 12 );
             }
             else if ( command.startsWith( "VFORK_EXEC " ) )
             {
                 processConfig.setStartType( WrapperProcessConfig.VFORK_EXEC );
                 type = "VFORK_EXEC";
-                command.substring( 11 );
+                command = command.substring( 11 );
             }
             else if ( command.startsWith( "DYNAMIC " ) )
             {
                 processConfig.setStartType( WrapperProcessConfig.DYNAMIC );
                 type = "DYNAMIC";
-                command.substring( 8 );
+                command = command.substring( 8 );
             }
             else
             {
@@ -713,15 +855,28 @@ public abstract class AbstractActionApp
         System.err.println( Main.getRes().getString( "   stopimmediate1           : Calls WrapperManager.stopImmediate(1)" ) );
         System.err.println( Main.getRes().getString( "  Actions which should cause the Wrapper to restart the JVM:" ) );
         System.err.println( Main.getRes().getString( "   access_violation_native  : Calls WrapperManager.accessViolationNative()" ) );
+        if ( WrapperManager.isWindows() )
+        {
+            System.err.println( Main.getRes().getString( "   exception_native         : Calls WrapperManager.raiseExceptionNative()" ) );
+            System.err.println( Main.getRes().getString( "   ff_exception_native      : Calls WrapperManager.raiseFailFastExceptionNative()" ) );
+        }
         System.err.println( Main.getRes().getString( "   appear_hung              : Calls WrapperManager.appearHung()" ) );
         System.err.println( Main.getRes().getString( "   halt0                    : Calls Runtime.getRuntime().halt(0)" ) );
         System.err.println( Main.getRes().getString( "   halt1                    : Calls Runtime.getRuntime().halt(1)" ) );
         System.err.println( Main.getRes().getString( "   restart                  : Calls WrapperManager.restart()" ) );
         System.err.println( Main.getRes().getString( "   restartandreturn         : Calls WrapperManager.restartAndReturn()" ) );
         System.err.println( Main.getRes().getString( "  Additional Tests:" ) );
+        System.err.println( Main.getRes().getString( "   appear_slow_1            : Calls WrapperManager.appearSlow(1)" ) );
+        System.err.println( Main.getRes().getString( "   appear_slow_5            : Calls WrapperManager.appearSlow(5)" ) );
+        System.err.println( Main.getRes().getString( "   appear_slow_reset        : Calls WrapperManager.appearSlow(0) to return to normal" ) );
+        System.err.println( Main.getRes().getString( "   deadlock                 : Executes some deadlocking code to test deadlock detection" ) );
+        System.err.println( Main.getRes().getString( "   outofmemory              : Simulates an OutOfMemoryError being thrown" ) );
         System.err.println( Main.getRes().getString( "   ignore_events            : Makes this application ignore control events." ) );
         System.err.println( Main.getRes().getString( "   dump                     : Calls WrapperManager.requestThreadDump()" ) );
-        System.err.println( Main.getRes().getString( "   deadlock_out             : Deadlocks the JVM's System.out and err streams." ) );
+        System.err.println( Main.getRes().getString( "   deadlock_out             : Deadlocks the JVM''s System.out and err streams." ) );
+        System.err.println( Main.getRes().getString( "   suspend_timeouts_10      : Calls WrapperManager.suspendTimeouts(10)  (Standard, Professional)" ) );
+        System.err.println( Main.getRes().getString( "   suspend_timeouts_60      : Calls WrapperManager.suspendTimeouts(60)  (Standard, Professional)" ) );
+        System.err.println( Main.getRes().getString( "   resume_timeouts          : Calls WrapperManager.resumeTimeouts()  (Standard, Professional)" ) );
         System.err.println( Main.getRes().getString( "   users                    : Start polling the current and interactive users." ) );
         System.err.println( Main.getRes().getString( "   groups                   : Start polling the current and interactive users with groups." ) );
         System.err.println( Main.getRes().getString( "   console                  : Prompt for actions in the console." ) );
@@ -731,6 +886,7 @@ public abstract class AbstractActionApp
         System.err.println( Main.getRes().getString( "   gc                       : Perform a GC sweep." ) );
         System.err.println( Main.getRes().getString( "   is_professional          : Displays whether or not this is a Professional Edition Wrapper." ) );
         System.err.println( Main.getRes().getString( "   is_standard              : Displays whether or not this is at least a Standard Edition Wrapper." ) );
+        System.err.println( Main.getRes().getString( "   current_time             : Prints the current time and timezone." ) );
         if ( WrapperManager.isProfessionalEdition() )
         {
             System.err.println( Main.getRes().getString( "   exec <cmd>               : Executes a managed child process." ) );
