@@ -3,7 +3,7 @@ package org.tanukisoftware.wrapper.test;
 import java.lang.reflect.InvocationTargetException;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -17,7 +17,7 @@ import java.lang.reflect.InvocationTargetException;
  * This is test is designed to simulate various deadlock cases using either
  * ReentrantLock instances or synchronized objects.
  * 
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class DeadLockBase
 {
@@ -93,16 +93,14 @@ public class DeadLockBase
             if ( rentrantLockClass.isInstance( m_lock2 ) )
             {
                 Object lock2 = m_lock2;
-                rentrantLockClass.getMethod( "lock", null ).invoke( lock2,
-                                                                    ( Object[] ) null );
+                rentrantLockClass.getMethod( "lock", (Class[])null ).invoke( lock2, (Object[])null );
                 try
                 {
                     secondLocked();
                 }
                 finally
                 {
-                    rentrantLockClass.getMethod( "unlock", null ).invoke( lock2,
-                                                                          ( Object[] ) null );
+                    rentrantLockClass.getMethod( "unlock", (Class[])null ).invoke( lock2, (Object[])null );
                 }
             }
             else
@@ -159,16 +157,14 @@ public class DeadLockBase
             try
             {
                 Object lock1 = m_lock1;
-                rentrantLockClass.getMethod( "lock", null ).invoke( lock1,
-                                                                    ( Object[] ) null );
+                rentrantLockClass.getMethod( "lock", (Class[])null ).invoke( lock1, (Object[])null );
                 try
                 {
                     firstLocked();
                 }
                 finally
                 {
-                    rentrantLockClass.getMethod( "unlock", null ).invoke( lock1,
-                                                                          ( Object[] ) null );
+                    rentrantLockClass.getMethod( "unlock", (Class[])null ).invoke( lock1, ( Object[] ) null );
                 }
             }
             catch ( NoSuchMethodException nsme )
