@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -12,6 +12,7 @@ package org.tanukisoftware.wrapper.test;
  */
 
 import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.io.PrintStream;
 
 /**
@@ -19,7 +20,7 @@ import java.io.PrintStream;
  *  to it will result in the calling threads deadlocking in the call.
  * Obviously, this class will not be useful to many as it is for tests.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class DeadlockPrintStream extends PrintStream {
     /** The Wrapped PrintStream. */
@@ -42,6 +43,22 @@ public class DeadlockPrintStream extends PrintStream {
         
         m_out = out;
     }
+
+    /**
+     * Creates a new DeadlockPrintStream wrapped around another PrintStream.
+     *
+     * @param out The PrintStream which will be wrapped by this new stream.
+     * @param autoFlush Whether the output buffer should be automatically flushed or not. ill be passed to the parent stream.
+     * @param encoding The name of a supported character encoding. Will be passed to the parent stream.
+     *
+     * @throws UnsupportedEncodingException If the named encoding is not supported.
+     */
+    public DeadlockPrintStream( PrintStream out, boolean autoFlush, String encoding ) throws UnsupportedEncodingException
+    {
+        super( out, autoFlush, encoding );
+        
+        m_out = out;
+    }
     
     /*---------------------------------------------------------------
      * PrintStream Methods
