@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -34,6 +34,7 @@ import java.awt.Button;
 import java.awt.Checkbox;
 import java.awt.Component;
 import java.awt.Container;
+import java.awt.Font;
 import java.awt.Frame;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
@@ -46,12 +47,23 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowListener;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JDialog;
+import javax.swing.BoxLayout;
+import javax.swing.border.Border;
+import javax.swing.border.EmptyBorder;
+import javax.swing.border.CompoundBorder;
 import org.tanukisoftware.wrapper.WrapperActionServer;
 import org.tanukisoftware.wrapper.WrapperListener;
 import org.tanukisoftware.wrapper.WrapperManager;
 import org.tanukisoftware.wrapper.WrapperResources;
 import org.tanukisoftware.wrapper.WrapperSystemPropertyUtil;
 import org.tanukisoftware.wrapper.event.WrapperEventListener;
+import org.tanukisoftware.wrapper.event.WrapperSecondInvocationEvent;
+import org.tanukisoftware.wrapper.event.WrapperServicePauseEvent;
+import org.tanukisoftware.wrapper.event.WrapperServiceResumeEvent;
+import org.tanukisoftware.wrapper.event.WrapperEvent;
 
 /**
  * This is a Test / Example program which can be used to test the
@@ -65,7 +77,7 @@ import org.tanukisoftware.wrapper.event.WrapperEventListener;
  *  see the <a href="http://wrapper.tanukisoftware.com/doc/english/integrate.html">integration</a>
  *  section of the documentation for more details.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class Main
     extends AbstractActionApp
@@ -79,11 +91,14 @@ public class Main
     private static WrapperResources m_res;
     private List m_listenerFlags;
     private TextField m_slowSeconds;
+    private TextField m_suspendSeconds;
     private TextField m_serviceName;
     private TextField m_consoleTitle;
     private TextField m_childCommand;
     private Checkbox m_childDetached;
     
+    private JDialog m_pauseDialog;
+    
     /*---------------------------------------------------------------
      * Constructors
      *-------------------------------------------------------------*/
@@ -99,15 +114,32 @@ public class Main
          * Serial Version UID.
          */
         private static final long serialVersionUID = -3847376282833547574L;
+        private float dpiScaleFactor = 1;
 
         MainFrame()
         {
             super( getRes().getString( "TestWrapper Example Application" ) );
             
+            try
+            {
+                // Rescale the GUI if the dpi awareness is 0 (not aware). Newer versions of Java are dpi aware and will automatically rescale the GUI.
+                if ( WrapperManager.nativeGetDpiAwareness() == 0 )
+                {
+                    // Set the dpi awareness to "per monitor" and get the dpi scale factor.
+                    WrapperManager.nativeSetDpiAwareness(2);
+                    dpiScaleFactor = (float)WrapperManager.nativeGetDpiScale() / 96;
+                }
+            }
+            catch ( UnsatisfiedLinkError e )
+            {
+                // This can happen if an old native library is used.  Fall through and make sure the scale factor is 1.
+                dpiScaleFactor = 1;
+            }
+            
             init();
             
             setLocation( 10, 10 );
-            setSize( 750, 480 );
+            setSize( (int)(750*dpiScaleFactor), (int)(480*dpiScaleFactor));
             
             setResizable( true );
         }
@@ -167,6 +199,15 @@ public class Main
             buildCommand( panel, gridBag, c, getRes().getString( "Native Access Violation" ), "access_violation_native",
                     getRes().getString( "Causes an access violation using native code, the JVM will crash and be restarted." ) );
             
+            if ( WrapperManager.isWindows() )
+            {
+                buildCommand( panel, gridBag, c, getRes().getString( "Native Exception" ), "exception_native",
+                        getRes().getString( "Throw an exception using native code, on Windows, the JVM will crash and be restarted." ) );
+                
+                buildCommand( panel, gridBag, c, getRes().getString( "Native Fail Fast Exception" ), "ff_exception_native",
+                        getRes().getString( "Throw a fail fast exception using native code, on Windows, the JVM will crash and be restarted." ) );
+            }
+            
             buildCommand( panel, gridBag, c, getRes().getString( "Simulate JVM Hang" ), "appear_hung",
                     getRes().getString( "Makes the JVM appear to be hung as viewed from the Wrapper, it will be killed and restarted." ) );
             
@@ -182,7 +223,7 @@ public class Main
             buildCommand( panel, gridBag, c, getRes().getString( "Simulate Slow JVM" ), "appear_slow", slowPanel2 );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Create Deadlock" ), "deadlock",
-                    getRes().getString( "Creates two new threads which intentionally go into a DeadLock situation.  (Standard, Professional)" ) );
+                    getRes().getString( "Creates two new threads which intentionally go into a deadlock situation.  (Standard, Professional)" ) );
             
             buildCommand( panel, gridBag, c, getRes().getString("Simulate Out Of Memory" ), "outofmemory",
                     getRes().getString( "Throws an OutOfMemoryError to demonstrate the Trigger feature." ) );
@@ -196,11 +237,25 @@ public class Main
             buildCommand( panel, gridBag, c, getRes().getString( "System.out Deadlock" ), "deadlock_out",
                     getRes().getString( "Simulates a failure mode where the System.out object has become deadlocked." ) );
             
+            m_suspendSeconds = new TextField( "60" );
+            Panel suspendPanel = new Panel();
+            suspendPanel.setLayout( new BorderLayout() );
+            suspendPanel.add( new Label( getRes().getString( "Seconds: " ) ), BorderLayout.WEST );
+            suspendPanel.add( m_suspendSeconds, BorderLayout.CENTER );
+            Panel suspendPanel2 = new Panel();
+            suspendPanel2.setLayout( new BorderLayout() );
+            suspendPanel2.add( suspendPanel, BorderLayout.WEST );
+            suspendPanel2.add( new Label( getRes().getString( "Suspend Wrapper timeouts while the JVM is running.  (Standard, Professional)" ) ), BorderLayout.CENTER );
+            buildCommand( panel, gridBag, c, getRes().getString( "Suspend Timeouts" ), "suspend_timeouts", suspendPanel2 );
+            
+            buildCommand( panel, gridBag, c, getRes().getString( "Resume Timeouts" ), "resume_timeouts",
+                    getRes().getString( "Resume Wrapper timeouts.  (Standard, Professional)" ) );
+            
             buildCommand( panel, gridBag, c, getRes().getString( "Poll Users" ), "users",
                     getRes().getString( "Begins calling WrapperManager.getUser() and getInteractiveUser() to monitor the current and interactive users." ) );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Poll Users with Groups" ), "groups",
-                    getRes().getString( "Same as above, but includes information about the user's groups." ) );
+                    getRes().getString( "Same as above, but includes information about the user''s groups." ) );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Console" ), "console", getRes().getString( "Prompt for Actions in the console." ) );
             
@@ -218,6 +273,7 @@ public class Main
             m_listenerFlags.add( "Control" );
             m_listenerFlags.add( "Logging" );
             m_listenerFlags.add( "Core" );
+            m_listenerFlags.add( "RemoteControl" );
             
             Panel flagPanel = new Panel();
             flagPanel.setLayout( new BorderLayout() );
@@ -277,7 +333,13 @@ public class Main
             
             buildCommand( panel, gridBag, c, getRes().getString( "Is Standard?" ), "is_standard", getRes().getString( "Prints true if this is a Standard Edition." ) );
             
+            buildCommand( panel, gridBag, c, getRes().getString( "Print time" ), "current_time", getRes().getString( "Prints the current time and timezone." ) );
+            
             addWindowListener( this );
+            
+            // By default listen to Service and RemoteControl events.
+            buildEventMask( new String[] { "Service", "RemoteControl" } );
+            updateEventListener();
         }
         
         private void buildCommand( Container container,
@@ -289,6 +351,9 @@ public class Main
         {
             Button button = new Button( label );
             button.setActionCommand( command );
+            if (dpiScaleFactor != 1) {
+                button.setFont(new Font("Arial", Font.PLAIN, (int)(12 * dpiScaleFactor)));
+            }
             
             c.fill = GridBagConstraints.BOTH;
             c.gridwidth = 1;
@@ -310,11 +375,46 @@ public class Main
             {
                 desc = new Label( description.toString() );
             }
+            if (dpiScaleFactor != 1) {
+                desc.setFont(new Font("Arial", Font.PLAIN, (int)(12 * dpiScaleFactor)));
+            }
             
             gridBag.setConstraints( desc, c );
             container.add( desc );
         }
         
+        private void buildEventMask(String[] flags) 
+        {
+            // Create the mask.
+            long mask = 0;
+            for ( int i = 0; i < flags.length; i++ )
+            {
+                String flag = flags[i];
+                if ( flag.equals( "Service" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_SERVICE;
+                }
+                else if ( flag.equals( "Control" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_CONTROL;
+                }
+                else if ( flag.equals( "Logging" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_LOGGING;
+                }
+                else if ( flag.equals( "Core" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_CORE;
+                }
+                else if ( flag.equals( "RemoteControl" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_REMOTE_CONTROL;
+                }
+            }
+            
+            setEventMask( mask );
+        }
+        
         /**************************************************************************
          * ActionListener Methods
          *************************************************************************/
@@ -323,31 +423,8 @@ public class Main
             String action = event.getActionCommand();
             if ( action.equals( "listener" ) )
             {
-                // Create the mask.
-                long mask = 0;
                 String[] flags = m_listenerFlags.getSelectedItems();
-                for ( int i = 0; i < flags.length; i++ )
-                {
-                    String flag = flags[i];
-                    if ( flag.equals( "Service" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_SERVICE;
-                    }
-                    else if ( flag.equals( "Control" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_CONTROL;
-                    }
-                    else if ( flag.equals( "Logging" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_LOGGING;
-                    }
-                    else if ( flag.equals( "Core" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_CORE;
-                    }
-                }
-                
-                setEventMask( mask );
+                buildEventMask(flags);
             }
             
             int slowSeconds;
@@ -361,6 +438,17 @@ public class Main
             }
             m_slowSeconds.setText( Integer.toString( slowSeconds ) );
             setSlowSeconds( slowSeconds );
+            int suspendSeconds;
+            try
+            {
+                suspendSeconds = Integer.parseInt( m_suspendSeconds.getText() );
+            }
+            catch ( NumberFormatException e )
+            {
+                suspendSeconds = 0;
+            }
+            m_suspendSeconds.setText( Integer.toString( suspendSeconds ) );
+            setSuspendSeconds( suspendSeconds );
             setServiceName( m_serviceName.getText() );
             setConsoleTitle( m_consoleTitle.getText() );
             setChildParams( m_childCommand.getText(), m_childDetached.getState() );
@@ -437,6 +525,42 @@ public class Main
         }
     }
     
+    /*---------------------------------------------------------------
+     * WrapperEventListener Methods
+     *-------------------------------------------------------------*/
+    /**
+     * Called whenever a WrapperEvent is fired.  The exact set of events that a
+     *  listener will receive will depend on the mask supplied when
+     *  WrapperManager.addWrapperEventListener was called to register the
+     *  listener.
+     *
+     * Listener implementations should never assume that they will only receive
+     *  events of a particular type.   To assure that events added to future
+     *  versions of the Wrapper do not cause problems with user code, events
+     *  should always be tested with "if ( event instanceof {EventClass} )"
+     *  before casting it to a specific event type.
+     *
+     * @param event WrapperEvent which was fired.
+     */
+    public void fired( WrapperEvent event )
+    {
+        if (event instanceof WrapperSecondInvocationEvent )
+        {
+            // bring the window to front
+            bringToFront(m_frame);
+        }
+        if (event instanceof WrapperServicePauseEvent )
+        {
+            // pause the demo application
+            pause();
+        }
+        if (event instanceof WrapperServiceResumeEvent )
+        {
+            // resume the demo application
+            resume();
+        }
+    }
+
     /*---------------------------------------------------------------
      * WrapperListener Methods
      *-------------------------------------------------------------*/
@@ -485,14 +609,12 @@ public class Main
             System.out.println( getRes().getString( "TestWrapper:     U: Unexpected Halt (Simulate crash)" ) );
             System.out.println( getRes().getString( "TestWrapper:     V: Access Violation (Actual crash)" ) );
             System.out.println( getRes().getString( "TestWrapper:     G: Make the JVM appear to be hung." ) );
-            System.out.println( "TestWrapper:" );
+            System.out.println( "TestWrapper: " );
         }
         catch ( java.io.IOException e )
         {
             System.out.println( getRes().getString( "TestWrapper: Unable to open the action server socket: {0}", e.getMessage() ) );
-            System.out.println( "TestWrapper:" );
-            m_actionServer = null;
-            System.out.println( "TestWrapper:" );
+            System.out.println( "TestWrapper: " );
             m_actionServer = null;
         }
         
@@ -588,6 +710,66 @@ public class Main
         return exitCode;
     }
     
+    /**
+     * Pause the demo application. 
+     */
+    private void pause() {
+        // Update the title with the "paused" state.
+        m_frame.setTitle( m_frame.getTitle() + getRes().getString( " (paused)" ) );
+        
+        // Disable all components of the frame.
+        enableComponents( m_frame, false );
+        
+        // Show a dialog centered over the frame to indicate that the application was paused and is waiting the resume event.
+        JLabel label1 = new JLabel();
+        label1.setHorizontalAlignment( javax.swing.SwingConstants.CENTER );
+        label1.setAlignmentX(Component.CENTER_ALIGNMENT);
+        label1.setText( getRes().getString( "The demo application is paused." ) );
+        
+        JLabel label2 = new JLabel();
+        label2.setHorizontalAlignment( javax.swing.SwingConstants.CENTER );
+        label2.setAlignmentX(Component.CENTER_ALIGNMENT);
+        label2.setText( getRes().getString( "Resume it to make this dialog box disappear." ) );
+      
+        Border border1 = label1.getBorder();
+        Border margin1 = new EmptyBorder( 25, 25, 0, 25 );
+        label1.setBorder( new CompoundBorder(border1, margin1) );
+      
+        Border border2 = label2.getBorder();
+        Border margin2 = new EmptyBorder( 0, 25, 30, 25 );
+        label2.setBorder( new CompoundBorder(border2, margin2) );
+        
+        JPanel panel = new JPanel();
+        panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
+        panel.add( label1 );
+        panel.add( label2 );
+
+        m_pauseDialog = new JDialog( m_frame, m_frame.getTitle(), false );
+        m_pauseDialog.getContentPane().add( panel );
+        m_pauseDialog.setResizable( false );
+        m_pauseDialog.setDefaultCloseOperation( javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE );
+        m_pauseDialog.pack();
+        m_pauseDialog.setLocationRelativeTo( m_frame );
+        m_pauseDialog.setVisible( true );
+    }
+
+    /**
+     * Resume the application. 
+     */
+    private void resume() {
+        String title = m_frame.getTitle();
+        // Update the title to remove the "paused" state.
+        m_frame.setTitle( title.substring( 0, title.indexOf( getRes().getString( " (paused)" ) ) ) );
+
+        // Enable all components of the frame.
+        enableComponents( m_frame, true );
+        
+        // Close the dialog.
+        m_pauseDialog.setDefaultCloseOperation( javax.swing.WindowConstants.DISPOSE_ON_CLOSE );
+        m_pauseDialog.setVisible(false);
+        m_pauseDialog.dispatchEvent(new WindowEvent(m_pauseDialog, WindowEvent.WINDOW_CLOSING));
+    }
+    
     public void controlEvent( int event )
     {
         System.out.println( getRes().getString( "TestWrapper: controlEvent({0})",  new Integer( event ) ) );
@@ -672,6 +854,36 @@ public class Main
         return m_res;
     }
     
+    /**
+     * Helper method to bring a window to the front
+     */
+    private boolean bringToFront(MainFrame frame) {
+        try {
+            // use reflection as setAlwaysOnTop() requires java 1.5
+            java.lang.reflect.Method m = MainFrame.class.getMethod("setAlwaysOnTop", new Class[] { boolean.class } );
+            m.invoke(frame, new Object[] { new Boolean(true) });
+            m.invoke(frame, new Object[] { new Boolean(false) });
+            return true;
+        } 
+        catch (Exception e)
+        {
+            return false;
+        }
+    }
+    
+    /**
+     * Helper method to disable or enable all components of a container
+     */
+    private void enableComponents(Container container, boolean enable) {
+        Component[] components = container.getComponents();
+        for (int a = 0; a < components.length; a++) {
+            components[a].setEnabled( enable );
+            if (components[a] instanceof Container) {
+                enableComponents((Container)components[a], enable);
+            }
+        }
+    }
+    
     /*---------------------------------------------------------------
      * Main Method
      *-------------------------------------------------------------*/
