@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -33,7 +33,7 @@ package org.tanukisoftware.wrapper.test;
  * Tests Bug #531880 where % characters in the text output were causing
  *  the wrapper to crash.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class PercentOutput {
     /*---------------------------------------------------------------
@@ -44,8 +44,29 @@ public class PercentOutput {
         System.out.println( "%");
         System.out.println("%%");
         System.out.println("%s");
+        System.out.println("%S");
         System.out.println("%d");
         System.out.println("\\%s%%");
+        System.out.println("\\%S%%");
+        
+        // This is a case from a user crash
+        System.out.println("         and vg.foobar like '%SEARCHKEY=vendorid%'");
+        
+        // Lots more output mixed with various quotes.
+        //  This will test the code that causes the internal buffer to be expanded.
+        for ( int i = 0; i < 100; i++ )
+        {
+            StringBuffer sb = new StringBuffer();
+            sb.append( i );
+            sb.append( ": " );
+            for ( int j = 0; j < i; j++ )
+            {
+                sb.append( ":01234567890123456%s90123456789012345%d890123456789012345%S8901234%%789012%S5678901234%p789012345678" );
+            }
+            sb.append( ":END" );
+            System.out.println( sb.toString() );
+        }
+        
         System.out.println( Main.getRes().getString( "Test Complete..." ) );
     }
 }
