@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -18,7 +18,7 @@ import org.tanukisoftware.wrapper.WrapperListener;
  * This test is to make sure the Wrapper works correctly when stop or restart is
  *  called before the start method has completed.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class StopWhileStarting implements WrapperListener {
     /**************************************************************************
@@ -53,6 +53,17 @@ public class StopWhileStarting implements WrapperListener {
             System.out.println( Main.getRes().getString( "start() returns exit code 99.  Will restart due to on_exit configuration" ) );
             return new Integer( 99 );
             
+        case 5:
+            System.out.println( Main.getRes().getString( "start(). Please press ctrl+c" ) );
+            try
+            {
+                Thread.sleep( 30000 );
+            }
+            catch ( InterruptedException e )
+            {
+            }
+            break;
+            
         default:
             System.out.println( Main.getRes().getString( "start() request stop(0)") );
             WrapperManager.stop( 0 );
